@@ -6,7 +6,12 @@ source activate $CONDA_ENV
 set -v -e
 
 # Build numba extensions without silencing compile errors
-python setup.py build_ext -q --inplace
+if [[ "$(uname -s)" == *"Linux"* ]] && [[ "$(uname -p)" == *"86"* ]]; then
+    EXTRA_BUILD_EXT_FLAGS="--werror --wall"
+else
+    EXTRA_BUILD_EXT_FLAGS=""
+fi
+python setup.py build_ext -q --inplace --debug $EXTRA_BUILD_EXT_FLAGS
 # (note we don't install to avoid problems with extra long Windows paths
 #  during distutils-dependent tests -- e.g. test_pycc)
 
