@@ -5,6 +5,8 @@ source activate $CONDA_ENV
 # Make sure any error below is reported as such
 set -v -e
 
+# Ensure the README is correctly formatted
+if [ "$BUILD_DOC" == "yes" ]; then rstcheck README.rst; fi
 # Ensure that the documentation builds without warnings
 pushd docs
 if [ "$BUILD_DOC" == "yes" ]; then make SPHINXOPTS=-W clean html; fi
@@ -63,10 +65,14 @@ python -m numba.tests.test_runtests
 # Now run tests based on the changes identified via git
 NUMBA_ENABLE_CUDASIM=1 $SEGVCATCH python -m numba.runtests -b -v -g -m $TEST_NPROCS -- numba.tests
 
+# List the tests found
+echo "INFO: All discovered tests:"
+python -m numba.runtests -l
 
 # Now run the Numba test suite with slicing
 # Note that coverage is run from the checkout dir to match the "source"
 # directive in .coveragerc
+echo "INFO: Running slice of discovered tests: ($TEST_START_INDEX,None,$TEST_COUNT)"
 if [ "$RUN_COVERAGE" == "yes" ]; then
     export PYTHONPATH=.
     coverage erase
