@@ -17,8 +17,10 @@ from numba.core.ir_utils import (raise_on_unsupported_feature, warn_deprecated,
                                  check_and_legalize_ir, guard,
                                  dead_code_elimination, simplify_CFG,
                                  get_definition, remove_dels,
-                                 build_definitions, compute_cfg_from_blocks)
+                                 build_definitions, compute_cfg_from_blocks,
+                                 is_operator_or_getitem)
 from numba.core import postproc
+from llvmlite import binding as llvm
 
 
 @contextmanager
@@ -304,7 +306,7 @@ class ParforPass(FunctionPass):
             # parfor calls the compiler chain again with a string
             if not (config.DISABLE_PERFORMANCE_WARNINGS or
                     state.func_ir.loc.filename == '<string>'):
-                url = ("http://numba.pydata.org/numba-doc/latest/user/"
+                url = ("https://numba.pydata.org/numba-doc/latest/user/"
                        "parallel.html#diagnostics")
                 msg = ("\nThe keyword argument 'parallel=True' was specified "
                        "but no transformation for parallel execution was "
@@ -353,6 +355,7 @@ class NativeLowering(LoweringPass):
         calltypes = state.calltypes
         flags = state.flags
         metadata = state.metadata
+        pre_stats = llvm.passmanagers.dump_refprune_stats()
 
         msg = ("Function %s failed at nopython "
                "mode lowering" % (state.func_id.func_name,))
@@ -400,6 +403,10 @@ class NativeLowering(LoweringPass):
                 targetctx.insert_user_function(cfunc, fndesc, [library])
                 state['cr'] = _LowerResult(fndesc, call_helper,
                                            cfunc=cfunc, env=env)
+
+            # capture pruning stats
+            post_stats = llvm.passmanagers.dump_refprune_stats()
+            metadata['prune_stats'] = post_stats - pre_stats
         return True
 
 
@@ -454,7 +461,6 @@ class NoPythonBackend(LoweringPass):
             call_helper=lowered.call_helper,
             signature=signature,
             objectmode=False,
-            interpmode=False,
             lifted=state.lifted,
             fndesc=lowered.fndesc,
             environment=lowered.env,
@@ -508,15 +514,11 @@ class InlineOverloads(FunctionPass):
         while work_list:
             label, block = work_list.pop()
             for i, instr in enumerate(block.body):
+                # TO-DO: other statements (setitem)
                 if isinstance(instr, ir.Assign):
                     expr = instr.value
                     if isinstance(expr, ir.Expr):
-                        if expr.op == 'call':
-                            workfn = self._do_work_call
-                        elif expr.op == 'getattr':
-                            workfn = self._do_work_getattr
-                        else:
-                            continue
+                        workfn = self._do_work_expr
 
                         if guard(workfn, state, work_list, block, i, expr,
                                  inline_worker):
@@ -548,69 +550,73 @@ class InlineOverloads(FunctionPass):
             print(''.center(80, '-'))
         return True
 
-    def _do_work_getattr(self, state, work_list, block, i, expr, inline_worker):
+    def _get_attr_info(self, state, expr):
         recv_type = state.type_annotation.typemap[expr.value.name]
         recv_type = types.unliteral(recv_type)
         matched = state.typingctx.find_matching_getattr_template(
             recv_type, expr.attr,
         )
         if not matched:
-            return False
+            return None
+
         template = matched['template']
         if getattr(template, 'is_method', False):
             # The attribute template is representing a method.
             # Don't inline the getattr.
-            return False
+            return None
 
-        inline_type = getattr(template, '_inline', None)
-        if inline_type is None:
-            # inline not defined
-            return False
+        templates = [template]
         sig = typing.signature(matched['return_type'], recv_type)
         arg_typs = sig.args
-
-        if not inline_type.is_never_inline:
-            try:
-                impl = template._overload_func(recv_type)
-                if impl is None:
-                    raise Exception  # abort for this template
-            except Exception:
-                return False
-        else:
-            return False
-
         is_method = False
-        return self._run_inliner(
-            state, inline_type, sig, template, arg_typs, expr, i, impl, block,
-            work_list, is_method, inline_worker,
-        )
 
-    def _do_work_call(self, state, work_list, block, i, expr, inline_worker):
-        # try and get a definition for the call, this isn't always possible as
-        # it might be a eval(str)/part generated awaiting update etc. (parfors)
-        to_inline = None
-        try:
-            to_inline = state.func_ir.get_definition(expr.func)
-        except Exception:
-            return False
+        return templates, sig, arg_typs, is_method
 
-        # do not handle closure inlining here, another pass deals with that.
-        if getattr(to_inline, 'op', False) == 'make_function':
-            return False
+    def _get_callable_info(self, state, expr):
 
-        # check this is a known and typed function
-        try:
-            func_ty = state.type_annotation.typemap[expr.func.name]
-        except KeyError:
-            # e.g. Calls to CUDA Intrinsic have no mapped type so KeyError
-            return False
-        if not hasattr(func_ty, 'get_call_type'):
-            return False
+        def get_func_type(state, expr):
+            func_ty = None
+            if expr.op == 'call':
+                # check this is a known and typed function
+                try:
+                    func_ty = state.type_annotation.typemap[expr.func.name]
+                except KeyError:
+                    # e.g. Calls to CUDA Intrinsic have no mapped type
+                    # so KeyError
+                    return None
+                if not hasattr(func_ty, 'get_call_type'):
+                    return None
+
+            elif is_operator_or_getitem(expr):
+                func_ty = state.typingctx.resolve_value_type(expr.fn)
+            else:
+                return None
+
+            return func_ty
+
+        if expr.op == 'call':
+            # try and get a definition for the call, this isn't always
+            # possible as it might be a eval(str)/part generated
+            # awaiting update etc. (parfors)
+            to_inline = None
+            try:
+                to_inline = state.func_ir.get_definition(expr.func)
+            except Exception:
+                return None
+
+            # do not handle closure inlining here, another pass deals with that
+            if getattr(to_inline, 'op', False) == 'make_function':
+                return None
+
+        func_ty = get_func_type(state, expr)
+        if func_ty is None:
+            return None
 
         sig = state.type_annotation.calltypes[expr]
-        is_method = False
+        if not sig:
+            return None
 
-        # search the templates for this overload looking for "inline"
+        templates, arg_typs, is_method = None, None, False
         if getattr(func_ty, 'template', None) is not None:
             # @overload_method
             is_method = True
@@ -621,30 +627,53 @@ class InlineOverloads(FunctionPass):
             templates = getattr(func_ty, 'templates', None)
             arg_typs = sig.args
 
-        if templates is None:
-            return False
+        return templates, sig, arg_typs, is_method
 
-        impl = None
-        for template in templates:
-            inline_type = getattr(template, '_inline', None)
-            if inline_type is None:
-                # inline not defined
-                continue
-            if not inline_type.is_never_inline:
-                try:
-                    impl = template._overload_func(*arg_typs)
-                    if impl is None:
-                        raise Exception  # abort for this template
-                    break
-                except Exception:
+    def _do_work_expr(self, state, work_list, block, i, expr, inline_worker):
+
+        def select_template(templates, args):
+            if templates is None:
+                return None
+
+            impl = None
+            for template in templates:
+                inline_type = getattr(template, '_inline', None)
+                if inline_type is None:
+                    # inline not defined
                     continue
+                if args not in template._inline_overloads:
+                    # skip overloads not matching signature
+                    continue
+                if not inline_type.is_never_inline:
+                    try:
+                        impl = template._overload_func(*args)
+                        if impl is None:
+                            raise Exception  # abort for this template
+                        break
+                    except Exception:
+                        continue
+            else:
+                return None
+
+            return template, inline_type, impl
+
+        inlinee_info = None
+        if expr.op == 'getattr':
+            inlinee_info = self._get_attr_info(state, expr)
         else:
+            inlinee_info = self._get_callable_info(state, expr)
+
+        if not inlinee_info:
             return False
 
-        # at this point we know we maybe want to inline something and there's
-        # definitely something that could be inlined.
+        templates, sig, arg_typs, is_method = inlinee_info
+        inlinee = select_template(templates, arg_typs)
+        if inlinee is None:
+            return False
+        template, inlinee_type, impl = inlinee
+
         return self._run_inliner(
-            state, inline_type, sig, template, arg_typs, expr, i, impl, block,
+            state, inlinee_type, sig, template, arg_typs, expr, i, impl, block,
             work_list, is_method, inline_worker,
         )
 
