@@ -28,7 +28,6 @@ from numba.core.errors import (TypingError, UntypedAttributeError,
                                ForceLiteralArg, CompilerError)
 from numba.core.funcdesc import qualifying_prefix
 
-
 _logger = logging.getLogger(__name__)
 
 
@@ -268,7 +267,38 @@ class _BuildContainerConstraint(object):
 
 
 class BuildListConstraint(_BuildContainerConstraint):
-    container_type = types.List
+
+    def __init__(self, target, items, loc):
+        self.target = target
+        self.items = items
+        self.loc = loc
+
+    def __call__(self, typeinfer):
+        with new_error_context("typing of {0} at {1}",
+                               types.List, self.loc):
+            typevars = typeinfer.typevars
+            tsets = [typevars[i.name].get() for i in self.items]
+            if not tsets:
+                typeinfer.add_type(self.target,
+                                   types.List(types.undefined),
+                                   loc=self.loc)
+            else:
+                for typs in itertools.product(*tsets):
+                    unified = typeinfer.context.unify_types(*typs)
+                    if unified is not None:
+                        # pull out literals if available
+                        islit = [isinstance(x, types.Literal) for x in typs]
+                        iv = None
+                        if all(islit):
+                            iv = [x.literal_value for x in typs]
+                        typeinfer.add_type(self.target,
+                                           types.List(unified,
+                                                      initial_value=iv),
+                                           loc=self.loc)
+                    else:
+                        typeinfer.add_type(self.target,
+                                           types.LiteralList(typs),
+                                           loc=self.loc)
 
 
 class BuildSetConstraint(_BuildContainerConstraint):
@@ -277,26 +307,67 @@ class BuildSetConstraint(_BuildContainerConstraint):
 
 class BuildMapConstraint(object):
 
-    def __init__(self, target, items, loc):
+    def __init__(self, target, items, special_value, value_indexes, loc):
         self.target = target
         self.items = items
+        self.special_value = special_value
+        self.value_indexes = value_indexes
         self.loc = loc
 
     def __call__(self, typeinfer):
+
         with new_error_context("typing of dict at {0}", self.loc):
             typevars = typeinfer.typevars
+
+            # figure out what sort of dict is being dealt with
             tsets = [(typevars[k.name].getone(), typevars[v.name].getone())
                      for k, v in self.items]
+
             if not tsets:
                 typeinfer.add_type(self.target,
                                    types.DictType(types.undefined,
-                                                  types.undefined),
+                                                  types.undefined,
+                                                  self.special_value),
                                    loc=self.loc)
             else:
-                key_type, value_type = tsets[0]
-                typeinfer.add_type(self.target,
-                                   types.DictType(key_type, value_type),
-                                   loc=self.loc)
+                # all the info is known about the dict, if its
+                # str keys -> random heterogeneous values treat as literalstrkey
+                ktys = [x[0] for x in tsets]
+                vtys = [x[1] for x in tsets]
+                strkey = all([isinstance(x, types.StringLiteral) for x in ktys])
+                literalvty = all([isinstance(x, types.Literal) for x in vtys])
+                vt0 = types.unliteral(vtys[0])
+                # homogeneous values comes in the form of being able to cast
+                # all the other values in the ctor to the type of the first
+
+                def check(other):
+                    return typeinfer.context.can_convert(other, vt0) is not None
+                homogeneous = all([check(types.unliteral(x)) for x in vtys])
+                # Special cases:
+                # Single key:value in ctor, key is str, value is an otherwise
+                # illegal container type, e.g. LiteralStrKeyDict or
+                # List, there's no way to put this into a typed.Dict, so make it
+                # a LiteralStrKeyDict, same goes for LiteralList.
+                if len(vtys) == 1:
+                    valty = vtys[0]
+                    if isinstance(valty, (types.LiteralStrKeyDict,
+                                          types.List,
+                                          types.LiteralList)):
+                        homogeneous = False
+
+                if strkey and not homogeneous:
+                    resolved_dict = {x: y for x, y in zip(ktys, vtys)}
+                    ty = types.LiteralStrKeyDict(resolved_dict,
+                                                 self.value_indexes)
+                    typeinfer.add_type(self.target, ty, loc=self.loc)
+                else:
+                    init_value = self.special_value if literalvty else None
+                    key_type, value_type = tsets[0]
+                    typeinfer.add_type(self.target,
+                                       types.DictType(key_type,
+                                                      value_type,
+                                                      init_value),
+                                       loc=self.loc)
 
 
 class ExhaustIterConstraint(object):
@@ -666,12 +737,18 @@ class SetItemRefinement(object):
         if _is_array_not_precise(targetty):
             typeinfer.add_type(self.target.name, sig.args[0], loc=self.loc)
         # For Dict setitem
-        if isinstance(targetty, types.DictType) and not targetty.is_precise():
-            refined = targetty.refine(idxty, valty)
-            typeinfer.add_type(
-                self.target.name, refined,
-                loc=self.loc,
-            )
+        if isinstance(targetty, types.DictType):
+            if not targetty.is_precise():
+                refined = targetty.refine(idxty, valty)
+                typeinfer.add_type(
+                    self.target.name, refined,
+                    loc=self.loc,
+                )
+            elif isinstance(targetty, types.LiteralStrKeyDict):
+                typeinfer.add_type(
+                    self.target.name, types.DictType(idxty, valty),
+                    loc=self.loc,
+                )
 
 
 class SetItemConstraint(SetItemRefinement):
@@ -916,7 +993,7 @@ class TypeInferer(object):
 
     def _get_return_vars(self):
         rets = []
-        for blk in utils.itervalues(self.blocks):
+        for blk in self.blocks.values():
             inst = blk.terminator
             if isinstance(inst, ir.Return):
                 rets.append(inst.value)
@@ -942,7 +1019,7 @@ class TypeInferer(object):
             self.lock_type(var.name, typ, loc=None)
 
     def build_constraint(self):
-        for blk in utils.itervalues(self.blocks):
+        for blk in self.blocks.values():
             for inst in blk.body:
                 self.constrain_statement(inst)
 
@@ -1063,7 +1140,7 @@ precise type that can be inferred from the other variables. Whilst sometimes
 the type of empty lists can be inferred, this is not always the case, see this
 documentation for help:
 
-http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-untyped-list-problem
+https://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-untyped-list-problem
 """
             if offender is not None:
                 # This block deals with imprecise lists
@@ -1137,6 +1214,9 @@ http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-u
                 raise e
             else:
                 retty = None
+        else:
+            typdict = utils.UniqueDict(
+                typdict, **{v.name: retty for v in self._get_return_vars()})
 
         try:
             fntys = self.get_function_types(typdict)
@@ -1404,8 +1484,7 @@ http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-u
         """
         Ensure that builtins are not modified.
         """
-        if (gvar.name in ('range', 'xrange') and
-                gvar.value not in utils.RANGE_ITER_OBJECTS):
+        if gvar.name == 'range' and gvar.value is not range:
             bad = True
         elif gvar.name == 'slice' and gvar.value is not slice:
             bad = True
@@ -1527,12 +1606,7 @@ http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-u
 
     def typeof_expr(self, inst, target, expr):
         if expr.op == 'call':
-            if isinstance(expr.func, ir.Intrinsic):
-                sig = expr.func.type
-                self.add_type(target.name, sig.return_type, loc=inst.loc)
-                self.add_calltype(expr, sig)
-            else:
-                self.typeof_call(inst, target, expr)
+            self.typeof_call(inst, target, expr)
         elif expr.op in ('getiter', 'iternext'):
             self.typeof_intrinsic_call(inst, target, expr.op, expr.value)
         elif expr.op == 'exhaust_iter':
@@ -1592,8 +1666,12 @@ http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-u
                                             loc=inst.loc)
             self.constraints.append(constraint)
         elif expr.op == 'build_map':
-            constraint = BuildMapConstraint(target.name, items=expr.items,
-                                            loc=inst.loc)
+            constraint = BuildMapConstraint(
+                target.name,
+                items=expr.items,
+                special_value=expr.literal_value,
+                value_indexes=expr.value_indexes,
+                loc=inst.loc)
             self.constraints.append(constraint)
         elif expr.op == 'cast':
             self.constraints.append(Propagate(dst=target.name,
