@@ -16,26 +16,25 @@ import os
 import ctypes
 import weakref
 import functools
-import copy
 import warnings
 import logging
 import threading
+import asyncio
 from itertools import product
 from abc import ABCMeta, abstractmethod
 from ctypes import (c_int, byref, c_size_t, c_char, c_char_p, addressof,
-                    c_void_p, c_float)
+                    c_void_p, c_float, c_uint)
 import contextlib
 import importlib
 import numpy as np
 from collections import namedtuple, deque
 
 from numba import mviewbuf
-from numba.core import utils, errors, serialize, config
+from numba.core import utils, serialize, config
 from .error import CudaSupportError, CudaDriverError
 from .drvapi import API_PROTOTYPES
-from .drvapi import cu_occupancy_b2d_size
+from .drvapi import cu_occupancy_b2d_size, cu_stream_callback_pyobj
 from numba.cuda.cudadrv import enums, drvapi, _extras
-from numba.core.utils import longint as long
 from numba.cuda.envvars import get_numba_envvar
 
 
@@ -44,7 +43,13 @@ MIN_REQUIRED_CC = (2, 0)
 SUPPORTS_IPC = sys.platform.startswith('linux')
 
 
-def _make_logger():
+_py_decref = ctypes.pythonapi.Py_DecRef
+_py_incref = ctypes.pythonapi.Py_IncRef
+_py_decref.argtypes = [ctypes.py_object]
+_py_incref.argtypes = [ctypes.py_object]
+
+
+def make_logger():
     logger = logging.getLogger(__name__)
     # is logging configured?
     if not logger.hasHandlers():
@@ -186,7 +191,7 @@ def _getpid():
 ERROR_MAP = _build_reverse_error_map()
 
 MISSING_FUNCTION_ERRMSG = """driver missing function: %s.
-Requires CUDA 8.0 or above.
+Requires CUDA 9.0 or above.
 """
 
 
@@ -224,7 +229,7 @@ class Driver(object):
     def initialize(self):
         # lazily initialize logger
         global _logger
-        _logger = _make_logger()
+        _logger = make_logger()
 
         self.is_initialized = True
         try:
@@ -582,8 +587,8 @@ class BaseCUDAMemoryManager(object, metaclass=ABCMeta):
 
         :param size: Size of the allocation in bytes
         :type size: int
-        :param mapped: Whether the allocated memory should be mapped into the CUDA
-                       address space.
+        :param mapped: Whether the allocated memory should be mapped into the
+                       CUDA address space.
         :type mapped: bool
         :param portable: Whether the memory will be considered pinned by all
                          contexts, and not just the calling context.
@@ -606,7 +611,8 @@ class BaseCUDAMemoryManager(object, metaclass=ABCMeta):
         :type pointer: int
         :param size: The size of the region in bytes.
         :type size: int
-        :param mapped: Whether the region should also be mapped into device memory.
+        :param mapped: Whether the region should also be mapped into device
+                       memory.
         :type mapped: bool
         :return: A memory pointer instance that refers to the allocated
                  memory.
@@ -752,7 +758,7 @@ class HostOnlyCUDAMemoryManager(BaseCUDAMemoryManager):
         It is recommended that this method is not overridden by EMM Plugin
         implementations - instead, use the :class:`BaseCUDAMemoryManager`.
         """
-        if isinstance(pointer, (int, long)):
+        if isinstance(pointer, int):
             pointer = c_void_p(pointer)
 
         # possible flags are "portable" (between context)
@@ -783,6 +789,26 @@ class HostOnlyCUDAMemoryManager(BaseCUDAMemoryManager):
             return PinnedMemory(ctx, pointer, size, owner=owner,
                                 finalizer=finalizer)
 
+    def memallocmanaged(self, size, attach_global):
+        ptr = drvapi.cu_device_ptr()
+
+        def allocator():
+            flags = c_uint()
+            if attach_global:
+                flags = enums.CU_MEM_ATTACH_GLOBAL
+            else:
+                flags = enums.CU_MEM_ATTACH_HOST
+
+            driver.cuMemAllocManaged(byref(ptr), size, flags)
+
+        self._attempt_allocation(allocator)
+
+        finalizer = _alloc_finalizer(self, ptr, size)
+        ctx = weakref.proxy(self.context)
+        mem = ManagedMemory(ctx, ptr, size, finalizer=finalizer)
+        self.allocations[ptr.value] = mem
+        return mem.own()
+
     def reset(self):
         """Clears up all host memory (mapped and/or pinned) in the current
         context.
@@ -804,7 +830,28 @@ class HostOnlyCUDAMemoryManager(BaseCUDAMemoryManager):
             yield
 
 
-class NumbaCUDAMemoryManager(HostOnlyCUDAMemoryManager):
+class GetIpcHandleMixin:
+    """A class that provides a default implementation of ``get_ipc_handle()``.
+    """
+
+    def get_ipc_handle(self, memory):
+        """Open an IPC memory handle by using ``cuMemGetAddressRange`` to
+        determine the base pointer of the allocation. An IPC handle of type
+        ``cu_ipc_mem_handle`` is constructed and initialized with
+        ``cuIpcGetMemHandle``. A :class:`numba.cuda.IpcHandle` is returned,
+        populated with the underlying ``ipc_mem_handle``.
+        """
+        base, end = device_extents(memory)
+        ipchandle = drvapi.cu_ipc_mem_handle()
+        driver.cuIpcGetMemHandle(byref(ipchandle), base)
+        source_info = self.context.device.get_device_identity()
+        offset = memory.handle.value - base
+
+        return IpcHandle(memory, ipchandle, memory.size, source_info,
+                         offset=offset)
+
+
+class NumbaCUDAMemoryManager(GetIpcHandleMixin, HostOnlyCUDAMemoryManager):
     """Internal on-device memory management for Numba. This is implemented using
     the EMM Plugin interface, but is not part of the public API."""
 
@@ -834,16 +881,6 @@ class NumbaCUDAMemoryManager(HostOnlyCUDAMemoryManager):
         driver.cuMemGetInfo(byref(free), byref(total))
         return MemoryInfo(free=free.value, total=total.value)
 
-    def get_ipc_handle(self, memory):
-        base, end = device_extents(memory)
-        ipchandle = drvapi.cu_ipc_mem_handle()
-        driver.cuIpcGetMemHandle(byref(ipchandle), base)
-        source_info = self.context.device.get_device_identity()
-        offset = memory.handle.value - base
-
-        return IpcHandle(memory, ipchandle, memory.size, source_info,
-                         offset=offset)
-
     @property
     def interface_version(self):
         return _SUPPORTED_EMM_INTERFACE_VERSION
@@ -853,6 +890,7 @@ _SUPPORTED_EMM_INTERFACE_VERSION = 1
 
 _memory_manager = None
 
+
 def _ensure_memory_manager():
     global _memory_manager
 
@@ -870,6 +908,7 @@ def _ensure_memory_manager():
         raise RuntimeError("Failed to use memory manager from %s" %
                            config.CUDA_MEMORY_MANAGER)
 
+
 def set_memory_manager(mm_plugin):
     """Configure Numba to use an External Memory Management (EMM) Plugin. If
     the EMM Plugin version does not match one supported by this version of
@@ -987,6 +1026,7 @@ MemoryInfo = namedtuple("MemoryInfo", "free,total")
     Total device memory in bytes.
 """
 
+
 class Context(object):
     """
     This object wraps a CUDA Context resource.
@@ -1021,20 +1061,27 @@ class Context(object):
         """
         return self.memory_manager.get_memory_info()
 
-    def get_active_blocks_per_multiprocessor(self, func, blocksize, memsize, flags=None):
+    def get_active_blocks_per_multiprocessor(self, func, blocksize, memsize,
+                                             flags=None):
         """Return occupancy of a function.
         :param func: kernel for which occupancy is calculated
         :param blocksize: block size the kernel is intended to be launched with
-        :param memsize: per-block dynamic shared memory usage intended, in bytes"""
+        :param memsize: per-block dynamic shared memory usage intended, in bytes
+        """
 
         retval = c_int()
         if not flags:
-            driver.cuOccupancyMaxActiveBlocksPerMultiprocessor(byref(retval), func.handle, blocksize, memsize)
+            driver.cuOccupancyMaxActiveBlocksPerMultiprocessor(byref(retval),
+                                                               func.handle,
+                                                               blocksize,
+                                                               memsize)
         else:
-            driver.cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags(byref(retval), func.handle, blocksize, memsize, flags)
+            driver.cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags(
+                byref(retval), func.handle, blocksize, memsize, flags)
         return retval.value
 
-    def get_max_potential_block_size(self, func, b2d_func, memsize, blocksizelimit, flags=None):
+    def get_max_potential_block_size(self, func, b2d_func, memsize,
+                                     blocksizelimit, flags=None):
         """Suggest a launch configuration with reasonable occupancy.
         :param func: kernel for which occupancy is calculated
         :param b2d_func: function that calculates how much per-block dynamic
@@ -1042,20 +1089,25 @@ class Context(object):
                          Can also be the address of a C function.
                          Use `0` to pass `NULL` to the underlying CUDA API.
         :param memsize: per-block dynamic shared memory usage intended, in bytes
-        :param blocksizelimit: maximum block size the kernel is designed to handle"""
+        :param blocksizelimit: maximum block size the kernel is designed to
+                               handle
+        """
 
         gridsize = c_int()
         blocksize = c_int()
         b2d_cb = cu_occupancy_b2d_size(b2d_func)
         if not flags:
-            driver.cuOccupancyMaxPotentialBlockSize(byref(gridsize), byref(blocksize),
-                                                    func.handle,
-                                                    b2d_cb,
+            driver.cuOccupancyMaxPotentialBlockSize(byref(gridsize),
+                                                    byref(blocksize),
+                                                    func.handle, b2d_cb,
                                                     memsize, blocksizelimit)
         else:
-            driver.cuOccupancyMaxPotentialBlockSizeWithFlags(byref(gridsize), byref(blocksize),
-                                                             func.handle, b2d_cb,
-                                                             memsize, blocksizelimit, flags)
+            driver.cuOccupancyMaxPotentialBlockSizeWithFlags(byref(gridsize),
+                                                             byref(blocksize),
+                                                             func.handle,
+                                                             b2d_cb, memsize,
+                                                             blocksizelimit,
+                                                             flags)
         return (gridsize.value, blocksize.value)
 
     def prepare_for_use(self):
@@ -1073,8 +1125,8 @@ class Context(object):
 
     def pop(self):
         """
-        Pops this context off the current CPU thread. Note that this context must
-        be at the top of the context stack, otherwise an error will occur.
+        Pops this context off the current CPU thread. Note that this context
+        must be at the top of the context stack, otherwise an error will occur.
         """
         popped = driver.pop_active_context()
         assert popped.value == self.handle.value
@@ -1082,6 +1134,9 @@ class Context(object):
     def memalloc(self, bytesize):
         return self.memory_manager.memalloc(bytesize)
 
+    def memallocmanaged(self, bytesize, attach_global=True):
+        return self.memory_manager.memallocmanaged(bytesize, attach_global)
+
     def memhostalloc(self, bytesize, mapped=False, portable=False, wc=False):
         return self.memory_manager.memhostalloc(bytesize, mapped, portable, wc)
 
@@ -1122,7 +1177,7 @@ class Context(object):
             byref(can_access_peer),
             self.device.id,
             peer_device,
-            )
+        )
         return bool(can_access_peer)
 
     def create_module_ptx(self, ptx):
@@ -1498,7 +1553,7 @@ class IpcHandle(object):
             self.size,
             self.source_info,
             self.offset,
-            )
+        )
         return (serialize._rebuild_reduction, args)
 
     @classmethod
@@ -1530,11 +1585,11 @@ class MemoryPointer(object):
     :type pointer: ctypes.c_void_p
     :param size: The size of the allocation in bytes.
     :type size: int
-    :param owner: The owner is sometimes set by the internals of this class, or used for
-                  Numba's internal memory management. It should not be provided
-                  by an external user of the ``MemoryPointer`` class (e.g. from
-                  within an EMM Plugin); the default of `None` should always
-                  suffice.
+    :param owner: The owner is sometimes set by the internals of this class, or
+                  used for Numba's internal memory management. It should not be
+                  provided by an external user of the ``MemoryPointer`` class
+                  (e.g. from within an EMM Plugin); the default of `None`
+                  should always suffice.
     :type owner: NoneType
     :param finalizer: A function that is called when the buffer is to be freed.
     :type finalizer: function
@@ -1626,7 +1681,7 @@ class AutoFreePointer(MemoryPointer):
 
 
 class MappedMemory(AutoFreePointer):
-    """A memory pointer that owns a buffer on the host that is mapped into
+    """A memory pointer that refers to a buffer on the host that is mapped into
     device memory.
 
     :param context: The context in which the pointer was mapped.
@@ -1635,11 +1690,11 @@ class MappedMemory(AutoFreePointer):
     :type pointer: ctypes.c_void_p
     :param size: The size of the buffer in bytes.
     :type size: int
-    :param owner: The owner is sometimes set by the internals of this class, or used for
-                  Numba's internal memory management. It should not be provided
-                  by an external user of the ``MappedMemory`` class (e.g. from
-                  within an EMM Plugin); the default of `None` should always
-                  suffice.
+    :param owner: The owner is sometimes set by the internals of this class, or
+                  used for Numba's internal memory management. It should not be
+                  provided by an external user of the ``MappedMemory`` class
+                  (e.g. from within an EMM Plugin); the default of `None`
+                  should always suffice.
     :type owner: NoneType
     :param finalizer: A function that is called when the buffer is to be freed.
     :type finalizer: function
@@ -1703,6 +1758,41 @@ class PinnedMemory(mviewbuf.MemAlloc):
         return self
 
 
+class ManagedMemory(AutoFreePointer):
+    """A memory pointer that refers to a managed memory buffer (can be accessed
+    on both host and device).
+
+    :param context: The context in which the pointer was mapped.
+    :type context: Context
+    :param pointer: The address of the buffer.
+    :type pointer: ctypes.c_void_p
+    :param size: The size of the buffer in bytes.
+    :type size: int
+    :param owner: The owner is sometimes set by the internals of this class, or
+                  used for Numba's internal memory management. It should not be
+                  provided by an external user of the ``ManagedMemory`` class
+                  (e.g. from within an EMM Plugin); the default of `None`
+                  should always suffice.
+    :type owner: NoneType
+    :param finalizer: A function that is called when the buffer is to be freed.
+    :type finalizer: function
+    """
+
+    __cuda_memory__ = True
+
+    def __init__(self, context, pointer, size, owner=None, finalizer=None):
+        self.owned = owner
+        devptr = pointer
+        super().__init__(context, devptr, size, finalizer=finalizer)
+
+        # For buffer interface
+        self._buflen_ = self.size
+        self._bufptr_ = self.device_pointer.value
+
+    def own(self):
+        return ManagedOwnedPointer(weakref.proxy(self))
+
+
 class OwnedPointer(object):
     def __init__(self, memptr, view=None):
         self._mem = memptr
@@ -1738,6 +1828,10 @@ class MappedOwnedPointer(OwnedPointer, mviewbuf.MemAlloc):
     pass
 
 
+class ManagedOwnedPointer(OwnedPointer, mviewbuf.MemAlloc):
+    pass
+
+
 class Stream(object):
     def __init__(self, context, handle, finalizer, external=False):
         self.context = context
@@ -1754,7 +1848,8 @@ class Stream(object):
         default_streams = {
             drvapi.CU_STREAM_DEFAULT: "<Default CUDA stream on %s>",
             drvapi.CU_STREAM_LEGACY: "<Legacy default CUDA stream on %s>",
-            drvapi.CU_STREAM_PER_THREAD: "<Per-thread default CUDA stream on %s>",
+            drvapi.CU_STREAM_PER_THREAD:
+                "<Per-thread default CUDA stream on %s>",
         }
         ptr = self.handle.value or drvapi.CU_STREAM_DEFAULT
         if ptr in default_streams:
@@ -1780,6 +1875,64 @@ class Stream(object):
         yield self
         self.synchronize()
 
+    def add_callback(self, callback, arg):
+        """
+        Add a callback to a compute stream.
+        The user provided function is called from a driver thread once all
+        preceding stream operations are complete.
+
+        Callback functions are called from a CUDA driver thread, not from
+        the thread that invoked `add_callback`. No CUDA API functions may
+        be called from within the callback function.
+
+        The duration of a callback function should be kept short, as the
+        callback will block later work in the stream and may block other
+        callbacks from being executed.
+
+        Note: This function is marked as deprecated and may be replaced in a
+        future CUDA release.
+
+        :param callback: Callback function with arguments (stream, status, arg).
+        :param arg: User data to be passed to the callback function.
+        """
+        data = (self, callback, arg)
+        _py_incref(data)
+        driver.cuStreamAddCallback(self.handle, self._stream_callback, data, 0)
+
+    @staticmethod
+    @cu_stream_callback_pyobj
+    def _stream_callback(handle, status, data):
+        try:
+            stream, callback, arg = data
+            callback(stream, status, arg)
+        except Exception as e:
+            warnings.warn(f"Exception in stream callback: {e}")
+        finally:
+            _py_decref(data)
+
+    def async_done(self) -> asyncio.futures.Future:
+        """
+        Return an awaitable that resolves once all preceding stream operations
+        are complete.
+        """
+        loop = asyncio.get_running_loop() if utils.PYVERSION >= (3, 7) \
+            else asyncio.get_event_loop()
+        future = loop.create_future()
+
+        def resolver(future, status):
+            if future.done():
+                return
+            elif status == 0:
+                future.set_result(None)
+            else:
+                future.set_exception(Exception(f"Stream error {status}"))
+
+        def callback(stream, status, future):
+            loop.call_soon_threadsafe(resolver, future, status)
+
+        self.add_callback(callback, future)
+        return future
+
 
 class Event(object):
     def __init__(self, context, handle, finalizer=None):
@@ -1899,35 +2052,6 @@ class Function(object):
             flag = enums.CU_FUNC_CACHE_PREFER_NONE
         driver.cuFuncSetCacheConfig(self.handle, flag)
 
-    def configure(self, griddim, blockdim, sharedmem=0, stream=0):
-        while len(griddim) < 3:
-            griddim += (1,)
-
-        while len(blockdim) < 3:
-            blockdim += (1,)
-
-        inst = copy.copy(self)  # shallow clone the object
-        inst.griddim = griddim
-        inst.blockdim = blockdim
-        inst.sharedmem = sharedmem
-        if stream:
-            inst.stream = stream
-        else:
-            inst.stream = 0
-        return inst
-
-    def __call__(self, *args):
-        '''
-        *args -- Must be either ctype objects of DevicePointer instances.
-        '''
-        if self.stream:
-            streamhandle = self.stream.handle
-        else:
-            streamhandle = None
-
-        launch_kernel(self.handle, self.griddim, self.blockdim,
-                      self.sharedmem, streamhandle, args)
-
     @property
     def device(self):
         return self.module.context.device
@@ -1951,9 +2075,12 @@ class Function(object):
                         maxthreads=maxtpb)
 
 
-def launch_kernel(cufunc_handle, griddim, blockdim, sharedmem, hstream, args):
-    gx, gy, gz = griddim
-    bx, by, bz = blockdim
+def launch_kernel(cufunc_handle,
+                  gx, gy, gz,
+                  bx, by, bz,
+                  sharedmem,
+                  hstream,
+                  args):
 
     param_vals = []
     for arg in args:
@@ -2150,6 +2277,7 @@ def _workaround_for_datetime(obj):
         obj = obj.view(np.int64)
     return obj
 
+
 def host_pointer(obj, readonly=False):
     """Get host pointer from an obj.
 
@@ -2159,7 +2287,7 @@ def host_pointer(obj, readonly=False):
     it should not be changed until the operation which can be asynchronous
     completes.
     """
-    if isinstance(obj, (int, long)):
+    if isinstance(obj, int):
         return obj
 
     forcewritable = False
@@ -2169,6 +2297,7 @@ def host_pointer(obj, readonly=False):
     obj = _workaround_for_datetime(obj)
     return mviewbuf.memoryview_get_buffer(obj, forcewritable, readonly)
 
+
 def host_memory_extents(obj):
     "Returns (start, end) the start and end pointer of the array (half open)."
     obj = _workaround_for_datetime(obj)
@@ -2210,7 +2339,7 @@ def is_device_memory(obj):
     "__cuda_memory__" defined and its value evaluated to True.
 
     All CUDA memory object should also define an attribute named
-    "device_pointer" which value is an int(or long) object carrying the pointer
+    "device_pointer" which value is an int object carrying the pointer
     value of the device memory address.  This is not tested in this method.
     """
     return getattr(obj, '__cuda_memory__', False)
