@@ -1,94 +1,92 @@
 """
 Enum values for CUDA driver
 """
-from __future__ import print_function, absolute_import, division
 
 
-CUDA_SUCCESS                              = 0
-CUDA_ERROR_INVALID_VALUE                  = 1
-CUDA_ERROR_OUT_OF_MEMORY                  = 2
-CUDA_ERROR_NOT_INITIALIZED                = 3
-CUDA_ERROR_DEINITIALIZED                  = 4
-CUDA_ERROR_PROFILER_DISABLED              = 5
-CUDA_ERROR_PROFILER_NOT_INITIALIZED       = 6
-CUDA_ERROR_PROFILER_ALREADY_STARTED       = 7
-CUDA_ERROR_PROFILER_ALREADY_STOPPED       = 8
-CUDA_ERROR_NO_DEVICE                      = 100
-CUDA_ERROR_INVALID_DEVICE                 = 101
-CUDA_ERROR_INVALID_IMAGE                  = 200
-CUDA_ERROR_INVALID_CONTEXT                = 201
-CUDA_ERROR_CONTEXT_ALREADY_CURRENT        = 202
-CUDA_ERROR_MAP_FAILED                     = 205
-CUDA_ERROR_UNMAP_FAILED                   = 206
-CUDA_ERROR_ARRAY_IS_MAPPED                = 207
-CUDA_ERROR_ALREADY_MAPPED                 = 208
-CUDA_ERROR_NO_BINARY_FOR_GPU              = 209
-CUDA_ERROR_ALREADY_ACQUIRED               = 210
-CUDA_ERROR_NOT_MAPPED                     = 211
-CUDA_ERROR_NOT_MAPPED_AS_ARRAY            = 212
-CUDA_ERROR_NOT_MAPPED_AS_POINTER          = 213
-CUDA_ERROR_ECC_UNCORRECTABLE              = 214
-CUDA_ERROR_UNSUPPORTED_LIMIT              = 215
-CUDA_ERROR_CONTEXT_ALREADY_IN_USE         = 216
-CUDA_ERROR_INVALID_SOURCE                 = 300
-CUDA_ERROR_FILE_NOT_FOUND                 = 301
+CUDA_SUCCESS = 0
+CUDA_ERROR_INVALID_VALUE = 1
+CUDA_ERROR_OUT_OF_MEMORY = 2
+CUDA_ERROR_NOT_INITIALIZED = 3
+CUDA_ERROR_DEINITIALIZED = 4
+CUDA_ERROR_PROFILER_DISABLED = 5
+CUDA_ERROR_PROFILER_NOT_INITIALIZED = 6
+CUDA_ERROR_PROFILER_ALREADY_STARTED = 7
+CUDA_ERROR_PROFILER_ALREADY_STOPPED = 8
+CUDA_ERROR_NO_DEVICE = 100
+CUDA_ERROR_INVALID_DEVICE = 101
+CUDA_ERROR_INVALID_IMAGE = 200
+CUDA_ERROR_INVALID_CONTEXT = 201
+CUDA_ERROR_CONTEXT_ALREADY_CURRENT = 202
+CUDA_ERROR_MAP_FAILED = 205
+CUDA_ERROR_UNMAP_FAILED = 206
+CUDA_ERROR_ARRAY_IS_MAPPED = 207
+CUDA_ERROR_ALREADY_MAPPED = 208
+CUDA_ERROR_NO_BINARY_FOR_GPU = 209
+CUDA_ERROR_ALREADY_ACQUIRED = 210
+CUDA_ERROR_NOT_MAPPED = 211
+CUDA_ERROR_NOT_MAPPED_AS_ARRAY = 212
+CUDA_ERROR_NOT_MAPPED_AS_POINTER = 213
+CUDA_ERROR_ECC_UNCORRECTABLE = 214
+CUDA_ERROR_UNSUPPORTED_LIMIT = 215
+CUDA_ERROR_CONTEXT_ALREADY_IN_USE = 216
+CUDA_ERROR_INVALID_SOURCE = 300
+CUDA_ERROR_FILE_NOT_FOUND = 301
 CUDA_ERROR_SHARED_OBJECT_SYMBOL_NOT_FOUND = 302
-CUDA_ERROR_SHARED_OBJECT_INIT_FAILED      = 303
-CUDA_ERROR_OPERATING_SYSTEM               = 304
-CUDA_ERROR_INVALID_HANDLE                 = 400
-CUDA_ERROR_NOT_FOUND                      = 500
-CUDA_ERROR_NOT_READY                      = 600
-CUDA_ERROR_LAUNCH_FAILED                  = 700
-CUDA_ERROR_LAUNCH_OUT_OF_RESOURCES        = 701
-CUDA_ERROR_LAUNCH_TIMEOUT                 = 702
-CUDA_ERROR_LAUNCH_INCOMPATIBLE_TEXTURING  = 703
-CUDA_ERROR_PEER_ACCESS_ALREADY_ENABLED    = 704
-CUDA_ERROR_PEER_ACCESS_NOT_ENABLED        = 705
-CUDA_ERROR_PRIMARY_CONTEXT_ACTIVE         = 708
-CUDA_ERROR_CONTEXT_IS_DESTROYED           = 709
-CUDA_ERROR_ASSERT                         = 710
-CUDA_ERROR_TOO_MANY_PEERS                 = 711
+CUDA_ERROR_SHARED_OBJECT_INIT_FAILED = 303
+CUDA_ERROR_OPERATING_SYSTEM = 304
+CUDA_ERROR_INVALID_HANDLE = 400
+CUDA_ERROR_NOT_FOUND = 500
+CUDA_ERROR_NOT_READY = 600
+CUDA_ERROR_LAUNCH_FAILED = 700
+CUDA_ERROR_LAUNCH_OUT_OF_RESOURCES = 701
+CUDA_ERROR_LAUNCH_TIMEOUT = 702
+CUDA_ERROR_LAUNCH_INCOMPATIBLE_TEXTURING = 703
+CUDA_ERROR_PEER_ACCESS_ALREADY_ENABLED = 704
+CUDA_ERROR_PEER_ACCESS_NOT_ENABLED = 705
+CUDA_ERROR_PRIMARY_CONTEXT_ACTIVE = 708
+CUDA_ERROR_CONTEXT_IS_DESTROYED = 709
+CUDA_ERROR_ASSERT = 710
+CUDA_ERROR_TOO_MANY_PEERS = 711
 CUDA_ERROR_HOST_MEMORY_ALREADY_REGISTERED = 712
-CUDA_ERROR_HOST_MEMORY_NOT_REGISTERED     = 713
-CUDA_ERROR_HARDWARE_STACK_ERROR           = 714
-CUDA_ERROR_ILLEGAL_INSTRUCTION            = 715
-CUDA_ERROR_MISALIGNED_ADDRESS             = 716
-CUDA_ERROR_INVALID_ADDRESS_SPACE          = 717
-CUDA_ERROR_INVALID_PC                     = 718
-CUDA_ERROR_LAUNCH_FAILED                  = 719
-CUDA_ERROR_COOPERATIVE_LAUNCH_TOO_LARGE   = 720
-CUDA_ERROR_NOT_PERMITTED                  = 800
-CUDA_ERROR_NOT_SUPPORTED                  = 801
-CUDA_ERROR_UNKNOWN                        = 999
+CUDA_ERROR_HOST_MEMORY_NOT_REGISTERED = 713
+CUDA_ERROR_HARDWARE_STACK_ERROR = 714
+CUDA_ERROR_ILLEGAL_INSTRUCTION = 715
+CUDA_ERROR_MISALIGNED_ADDRESS = 716
+CUDA_ERROR_INVALID_ADDRESS_SPACE = 717
+CUDA_ERROR_INVALID_PC = 718
+CUDA_ERROR_LAUNCH_FAILED = 719
+CUDA_ERROR_COOPERATIVE_LAUNCH_TOO_LARGE = 720
+CUDA_ERROR_NOT_PERMITTED = 800
+CUDA_ERROR_NOT_SUPPORTED = 801
+CUDA_ERROR_UNKNOWN = 999
 
 
 # no preference for shared memory or L1 (default)
-CU_FUNC_CACHE_PREFER_NONE    = 0x00
+CU_FUNC_CACHE_PREFER_NONE = 0x00
 # prefer larger shared memory and smaller L1 cache
-CU_FUNC_CACHE_PREFER_SHARED  = 0x01
+CU_FUNC_CACHE_PREFER_SHARED = 0x01
 # prefer larger L1 cache and smaller shared memory
-CU_FUNC_CACHE_PREFER_L1      = 0x02
+CU_FUNC_CACHE_PREFER_L1 = 0x02
 # prefer equal sized L1 cache and shared memory
-CU_FUNC_CACHE_PREFER_EQUAL   = 0x03
+CU_FUNC_CACHE_PREFER_EQUAL = 0x03
 
 # Automatic scheduling
-CU_CTX_SCHED_AUTO          = 0x00
+CU_CTX_SCHED_AUTO = 0x00
 # Set spin as default scheduling
-CU_CTX_SCHED_SPIN          = 0x01
+CU_CTX_SCHED_SPIN = 0x01
 # Set yield as default scheduling
-CU_CTX_SCHED_YIELD         = 0x02
+CU_CTX_SCHED_YIELD = 0x02
 # Set blocking synchronization as default scheduling
 CU_CTX_SCHED_BLOCKING_SYNC = 0x04
 
-CU_CTX_SCHED_MASK          = 0x07
+CU_CTX_SCHED_MASK = 0x07
 
 # Support mapped pinned allocations
-CU_CTX_MAP_HOST            = 0x08
+CU_CTX_MAP_HOST = 0x08
 # Keep local memory allocation after launch
-CU_CTX_LMEM_RESIZE_TO_MAX  = 0x10
-
-CU_CTX_FLAGS_MASK          = 0x1f
+CU_CTX_LMEM_RESIZE_TO_MAX = 0x10
 
+CU_CTX_FLAGS_MASK = 0x1f
 
 
 # If set, host memory is portable between CUDA contexts.
@@ -116,14 +114,30 @@ CU_MEMHOSTREGISTER_PORTABLE = 0x01
 CU_MEMHOSTREGISTER_DEVICEMAP = 0x02
 
 
+# If set, managed memory is accessible from all streams on all devices.
+CU_MEM_ATTACH_GLOBAL = 0x01
+
+# If set on a platform where the device attribute
+# cudaDevAttrConcurrentManagedAccess is zero, then managed memory is
+# only accessible on the host (unless explicitly attached to a stream
+# with cudaStreamAttachMemAsync, in which case it can be used in kernels
+# launched on that stream).
+CU_MEM_ATTACH_HOST = 0x02
+
+# If set on a platform where the device attribute
+# cudaDevAttrConcurrentManagedAccess is zero, then managed memory accesses
+# on the associated device must only be from a single stream.
+CU_MEM_ATTACH_SINGLE = 0x04
+
+
 # Default event flag
-CU_EVENT_DEFAULT        = 0x0
+CU_EVENT_DEFAULT = 0x0
 # Event uses blocking synchronization
-CU_EVENT_BLOCKING_SYNC  = 0x1
+CU_EVENT_BLOCKING_SYNC = 0x1
 # Event will not record timing data
 CU_EVENT_DISABLE_TIMING = 0x2
 # Event is suitable for interprocess use. CU_EVENT_DISABLE_TIMING must be set
-CU_EVENT_INTERPROCESS   = 0x4
+CU_EVENT_INTERPROCESS = 0x4
 
 # The CUcontext on which a pointer was allocated or registered
 CU_POINTER_ATTRIBUTE_CONTEXT = 1
@@ -137,16 +151,15 @@ CU_POINTER_ATTRIBUTE_HOST_POINTER = 4
 CU_POINTER_ATTRIBUTE_P2P_TOKENS = 5
 
 # Host memory
-CU_MEMORYTYPE_HOST    = 0x01
+CU_MEMORYTYPE_HOST = 0x01
 # Device memory
-CU_MEMORYTYPE_DEVICE  = 0x02
+CU_MEMORYTYPE_DEVICE = 0x02
 # Array memory
-CU_MEMORYTYPE_ARRAY   = 0x03
+CU_MEMORYTYPE_ARRAY = 0x03
 # Unified device or host memory
 CU_MEMORYTYPE_UNIFIED = 0x04
 
 
-
 # Compiled device-class-specific device code
 # Applicable options: none
 CU_JIT_INPUT_CUBIN = 0
@@ -168,7 +181,6 @@ CU_JIT_INPUT_OBJECT = 3
 CU_JIT_INPUT_LIBRARY = 4
 
 
-
 # Max number of registers that a thread may use.
 # Option type: unsigned int
 # Applies to: compiler only
