@@ -1,7 +1,7 @@
 import math
-from numba.core import types, utils
-from numba.core.typing.templates import (AttributeTemplate, ConcreteTemplate,
-                                    signature, Registry)
+from numba.core import types
+from numba.core.typing.templates import ConcreteTemplate, signature, Registry
+
 
 registry = Registry()
 infer_global = registry.register_global
@@ -85,8 +85,25 @@ class Math_pow(ConcreteTemplate):
     ]
 
 
+@infer_global(math.frexp)
+class Math_frexp(ConcreteTemplate):
+    cases = [
+        signature(types.Tuple([types.float32, types.int32]), types.float32),
+        signature(types.Tuple([types.float64, types.int32]), types.float64),
+    ]
+
+
+@infer_global(math.ldexp)
+class Math_ldexp(ConcreteTemplate):
+    cases = [
+        signature(types.float32, types.float32, types.int32),
+        signature(types.float64, types.float64, types.int32),
+    ]
+
+
 @infer_global(math.isinf)
 @infer_global(math.isnan)
+@infer_global(math.isfinite)
 class Math_isnan(ConcreteTemplate):
     cases = [
         signature(types.boolean, types.int64),
@@ -95,6 +112,7 @@ class Math_isnan(ConcreteTemplate):
         signature(types.boolean, types.float64),
     ]
 
+
 @infer_global(math.modf)
 class Math_modf(ConcreteTemplate):
     cases = [
