@@ -1,6 +1,5 @@
-from __future__ import print_function, division, absolute_import
-from numba.targets.descriptors import TargetDescriptor
-from numba.targets.options import TargetOptions
+from numba.core.descriptors import TargetDescriptor
+from numba.core.options import TargetOptions
 from .target import CUDATargetContext, CUDATypingContext
 
 
