@@ -1,7 +1,8 @@
 from numba import cuda
 from numba.cuda.cudadrv.driver import driver
-from numba import numpy_support as nps
 import math
+from numba.np import numpy_support as nps
+
 
 def transpose(a, b=None):
     """Compute the transpose of 'a' and store it into 'b', if given,
@@ -28,14 +29,14 @@ def transpose(a, b=None):
             dtype=a.dtype,
             stream=stream)
 
-    dt=nps.from_dtype(a.dtype)
+    dt = nps.from_dtype(a.dtype)
 
     tpb = driver.get_device().MAX_THREADS_PER_BLOCK
     # we need to factor available threads into x and y axis
-    tile_width = int(math.pow(2, math.log(tpb, 2)/2))
+    tile_width = int(math.pow(2, math.log(tpb, 2) / 2))
     tile_height = int(tpb / tile_width)
 
-    tile_shape=(tile_height, tile_width + 1)
+    tile_shape = (tile_height, tile_width + 1)
 
     @cuda.jit
     def kernel(input, output):
@@ -49,15 +50,14 @@ def transpose(a, b=None):
         x = by + tx
         y = bx + ty
 
-        if by+ty < input.shape[0] and bx+tx < input.shape[1]:
-            tile[ty, tx] = input[by+ty, bx+tx]
+        if by + ty < input.shape[0] and bx + tx < input.shape[1]:
+            tile[ty, tx] = input[by + ty, bx + tx]
         cuda.syncthreads()
         if y < output.shape[0] and x < output.shape[1]:
             output[y, x] = tile[tx, ty]
 
-
     # one block per tile, plus one for remainders
-    blocks = int(b.shape[0]/tile_height + 1), int(b.shape[1]/tile_width + 1)
+    blocks = int(b.shape[0] / tile_height + 1), int(b.shape[1] / tile_width + 1)
     # one thread per tile element
     threads = tile_height, tile_width
     kernel[blocks, threads, stream](a, b)
