@@ -1,3 +1,9 @@
+import numpy as np
+from collections import namedtuple
+
+_MemoryInfo = namedtuple("_MemoryInfo", "free,total")
+
+
 class FakeCUDAContext(object):
     '''
     This stub implements functionality only for simulating a single GPU
@@ -10,7 +16,7 @@ class FakeCUDAContext(object):
         pass
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-       pass
+        pass
 
     def __str__(self):
         return "<Managed Device {self.id}>".format(self=self)
@@ -23,6 +29,29 @@ class FakeCUDAContext(object):
     def compute_capability(self):
         return (5, 2)
 
+    def reset(self):
+        pass
+
+    def get_memory_info(self):
+        """
+        Cross-platform free / total host memory is hard without external
+        dependencies, e.g. `psutil` - so return infinite memory to maintain API
+        type compatibility
+        """
+        return _MemoryInfo(float('inf'), float('inf'))
+
+    def memalloc(self, sz):
+        """
+        Allocates memory on the simulated device
+        At present, there is no division between simulated
+        host memory and simulated device memory.
+        """
+        return np.ndarray(sz, dtype='u1')
+
+    def memhostalloc(self, sz, mapped=False, portable=False, wc=False):
+        '''Allocates memory on the host'''
+        return self.memalloc(sz)
+
 
 class FakeDeviceList(object):
     '''
@@ -61,6 +90,10 @@ def reset():
     gpus[0].closed = True
 
 
+def get_context(devnum=0):
+    return FakeCUDAContext(devnum)
+
+
 def require_context(func):
     '''
     In the simulator, a context is always "available", so this is a no-op.
