@@ -1,10 +1,11 @@
 import re
-from llvmlite.llvmpy.core import (Type, Builder, LINKAGE_INTERNAL,
-                       Constant, ICMP_EQ)
+from llvmlite.llvmpy.core import (Type, Builder, LINKAGE_INTERNAL, Constant,
+                                  ICMP_EQ)
 import llvmlite.llvmpy.core as lc
 import llvmlite.binding as ll
 
-from numba.core import typing, types, dispatcher, debuginfo, itanium_mangler, cgutils
+from numba.core import (typing, types, dispatcher, debuginfo, itanium_mangler,
+                        cgutils)
 from numba.core.utils import cached_property
 from numba.core.base import BaseContext
 from numba.core.callconv import MinimalCallConv
@@ -22,11 +23,12 @@ from numba.cpython import cmathimpl
 
 class CUDATypingContext(typing.BaseContext):
     def load_additional_registries(self):
-        from . import cudadecl, cudamath
+        from . import cudadecl, cudamath, libdevicedecl
 
         self.install_registry(cudadecl.registry)
         self.install_registry(cudamath.registry)
         self.install_registry(cmathdecl.registry)
+        self.install_registry(libdevicedecl.registry)
 
     def resolve_value_type(self, val):
         # treat dispatcher object as another device function
@@ -38,7 +40,9 @@ class CUDATypingContext(typing.BaseContext):
                 if not val._can_compile:
                     raise ValueError('using cpu function on device '
                                      'but its compilation is disabled')
-                jd = jitdevice(val, debug=val.targetoptions.get('debug'))
+                opt = val.targetoptions.get('opt', True)
+                jd = jitdevice(val, debug=val.targetoptions.get('debug'),
+                               opt=opt)
                 # cache the device function for future use and to avoid
                 # duplicated copy of the same function.
                 val.__cudajitdevice = jd
@@ -50,6 +54,7 @@ class CUDATypingContext(typing.BaseContext):
 # -----------------------------------------------------------------------------
 # Implementation
 
+
 VALID_CHARS = re.compile(r'[^a-z0-9]', re.I)
 
 
@@ -72,11 +77,12 @@ class CUDATargetContext(BaseContext):
         self._target_data = ll.create_target_data(nvvm.default_data_layout)
 
     def load_additional_registries(self):
-        from . import cudaimpl, printimpl, libdevice
+        from . import cudaimpl, printimpl, libdeviceimpl, mathimpl
         self.install_registry(cudaimpl.registry)
         self.install_registry(printimpl.registry)
-        self.install_registry(libdevice.registry)
+        self.install_registry(libdeviceimpl.registry)
         self.install_registry(cmathimpl.registry)
+        self.install_registry(mathimpl.registry)
 
     def codegen(self):
         return self._internal_codegen
@@ -95,8 +101,8 @@ class CUDATargetContext(BaseContext):
         from numba import cuda
         nonconsts = ('threadIdx', 'blockDim', 'blockIdx', 'gridDim', 'laneid',
                      'warpsize')
-        nonconsts_with_mod = tuple([ (types.Module(cuda), nc)
-                                    for nc in nonconsts ])
+        nonconsts_with_mod = tuple([(types.Module(cuda), nc)
+                                    for nc in nonconsts])
         return nonconsts_with_mod
 
     @cached_property
@@ -133,7 +139,8 @@ class CUDATargetContext(BaseContext):
         wrapfnty = Type.function(Type.void(), argtys)
         wrapper_module = self.create_module("cuda.kernel.wrapper")
         fnty = Type.function(Type.int(),
-                             [self.call_conv.get_return_type(types.pyobject)] + argtys)
+                             [self.call_conv.get_return_type(types.pyobject)]
+                             + argtys)
         func = wrapper_module.add_function(fnty, name=fname)
 
         prefixed = itanium_mangler.prepend_namespace(func.name, ns='cudapy')
@@ -158,7 +165,6 @@ class CUDATargetContext(BaseContext):
         status, _ = self.call_conv.call_function(
             builder, func, types.void, argtypes, callargs)
 
-
         if debug:
             # Check error status
             with cgutils.if_likely(builder, status.is_ok):
@@ -172,7 +178,7 @@ class CUDATargetContext(BaseContext):
                 # Only the first error is recorded
 
                 casfnty = lc.Type.function(old.type, [gv_exc.type, old.type,
-                                                    old.type])
+                                                      old.type])
 
                 casfn = wrapper_module.add_function(casfnty,
                                                     name="___numba_cas_hack")
@@ -234,8 +240,8 @@ class CUDATargetContext(BaseContext):
         kshape = [self.get_constant(types.intp, s) for s in arr.shape]
         kstrides = [self.get_constant(types.intp, s) for s in arr.strides]
         self.populate_array(ary, data=builder.bitcast(genptr, ary.data.type),
-                            shape=cgutils.pack_array(builder, kshape),
-                            strides=cgutils.pack_array(builder, kstrides),
+                            shape=kshape,
+                            strides=kstrides,
                             itemsize=ary.itemsize, parent=ary.parent,
                             meminfo=None)
 
