@@ -1,5 +1,6 @@
 import contextlib
 import os
+import shutil
 import sys
 
 from numba.tests.support import (
@@ -59,6 +60,16 @@ def skip_under_cuda_memcheck(reason):
     return unittest.skipIf(os.environ.get('CUDA_MEMCHECK') is not None, reason)
 
 
+def skip_without_nvdisasm(reason):
+    nvdisasm_path = shutil.which('nvdisasm')
+    return unittest.skipIf(nvdisasm_path is None, reason)
+
+
+def skip_with_nvdisasm(reason):
+    nvdisasm_path = shutil.which('nvdisasm')
+    return unittest.skipIf(nvdisasm_path is not None, reason)
+
+
 class CUDATextCapture(object):
 
     def __init__(self, stream):
