@@ -1,5 +1,6 @@
 from ctypes import byref, c_int, sizeof
-from numba.cuda.cudadrv.driver import host_to_device, device_to_host, driver
+from numba.cuda.cudadrv.driver import (host_to_device, device_to_host, driver,
+                                       launch_kernel)
 from numba.cuda.cudadrv import devices, drvapi
 from numba.cuda.testing import unittest, CUDATestCase
 from numba.cuda.testing import skip_on_cudasim
@@ -82,8 +83,12 @@ class TestCudaDriver(CUDATestCase):
 
         host_to_device(memory, array, sizeof(array))
 
-        function = function.configure((1,), (100,))
-        function(memory)
+        launch_kernel(function.handle,  # Kernel
+                      1,   1, 1,        # gx, gy, gz
+                      100, 1, 1,        # bx, by, bz
+                      0,                # dynamic shared mem
+                      0,                # stream
+                      [memory])         # arguments
 
         device_to_host(array, memory, sizeof(array))
         for i, v in enumerate(array):
@@ -103,8 +108,12 @@ class TestCudaDriver(CUDATestCase):
             memory = self.context.memalloc(sizeof(array))
             host_to_device(memory, array, sizeof(array), stream=stream)
 
-            function = function.configure((1,), (100,), stream=stream)
-            function(memory)
+            launch_kernel(function.handle,  # Kernel
+                          1,   1, 1,        # gx, gy, gz
+                          100, 1, 1,        # bx, by, bz
+                          0,                # dynamic shared mem
+                          stream.handle,    # stream
+                          [memory])         # arguments
 
         device_to_host(array, memory, sizeof(array), stream=stream)
 
@@ -168,13 +177,18 @@ class TestCudaDriver(CUDATestCase):
         module = self.context.create_module_ptx(self.ptx)
         function = module.get_function('_Z10helloworldPi')
 
-        value = self.context.get_active_blocks_per_multiprocessor(function, 128, 128)
+        value = self.context.get_active_blocks_per_multiprocessor(function,
+                                                                  128, 128)
         self.assertTrue(value > 0)
-        def b2d(bs): return bs
-        grid, block = self.context.get_max_potential_block_size(function, b2d, 128, 128)
+
+        def b2d(bs):
+            return bs
+
+        grid, block = self.context.get_max_potential_block_size(function, b2d,
+                                                                128, 128)
         self.assertTrue(grid > 0)
         self.assertTrue(block > 0)
 
+
 if __name__ == '__main__':
     unittest.main()
-
