@@ -1,15 +1,12 @@
-from __future__ import print_function, division, absolute_import
-
 from llvmlite.llvmpy.core import Module, Type, Builder, InlineAsm
-from llvmlite import binding as ll
 
 from numba.cuda.cudadrv import nvvm
-from numba.cuda.testing import unittest, CUDATestCase
+from numba.cuda.testing import unittest, ContextResettingTestCase
 from numba.cuda.testing import skip_on_cudasim
 
 
 @skip_on_cudasim('Inline PTX cannot be used in the simulator')
-class TestCudaInlineAsm(CUDATestCase):
+class TestCudaInlineAsm(ContextResettingTestCase):
     def test_inline_rsqrt(self):
         mod = Module(__name__)
         fnty = Type.function(Type.void(), [Type.pointer(Type.float())])
