@@ -1,15 +1,13 @@
-from __future__ import print_function, division, absolute_import
-
 import numpy as np
 
 from numba import cuda
-from numba.cuda.testing import unittest, CUDATestCase
+from numba.cuda.testing import unittest, ContextResettingTestCase
 
 
 REPEAT = 25
 
 
-class TestPinned(CUDATestCase):
+class TestPinned(ContextResettingTestCase):
 
     def _run_copies(self, A):
         A0 = np.copy(A)
@@ -23,15 +21,14 @@ class TestPinned(CUDATestCase):
         self.assertTrue(np.allclose(A, A0))
 
     def test_pinned(self):
-        A = np.arange(2*1024*1024) # 16 MB
+        A = np.arange(2 * 1024 * 1024) # 16 MB
         with cuda.pinned(A):
             self._run_copies(A)
 
     def test_unpinned(self):
-        A = np.arange(2*1024*1024) # 16 MB
+        A = np.arange(2 * 1024 * 1024) # 16 MB
         self._run_copies(A)
 
 
 if __name__ == '__main__':
     unittest.main()
-
