@@ -1,19 +1,16 @@
-from __future__ import print_function, division, absolute_import
-
 import numpy as np
 
 from numba import cuda
-from numba.cuda.testing import unittest, SerialMixin
+from numba.cuda.testing import unittest, CUDATestCase
 
 
-class TestCudaArrayArg(SerialMixin, unittest.TestCase):
+class TestCudaArrayArg(CUDATestCase):
     def test_array_ary(self):
 
         @cuda.jit('double(double[:],int64)', device=True, inline=True)
         def device_function(a, c):
             return a[c]
 
-
         @cuda.jit('void(double[:],double[:])')
         def kernel(x, y):
             i = cuda.grid(1)
