@@ -1,10 +1,7 @@
-from __future__ import print_function, absolute_import, division
-
 import numpy as np
 import math
-import time
 from numba import cuda, double
-from numba.cuda.testing import unittest, SerialMixin
+from numba.cuda.testing import unittest, CUDATestCase
 
 
 RISKFREE = 0.02
@@ -47,7 +44,7 @@ def randfloat(rand_var, low, high):
     return (1.0 - rand_var) * low + rand_var * high
 
 
-class TestBlackScholes(SerialMixin, unittest.TestCase):
+class TestBlackScholes(CUDATestCase):
     def test_blackscholes(self):
         OPT_N = 400
         iterations = 2
@@ -67,8 +64,6 @@ class TestBlackScholes(SerialMixin, unittest.TestCase):
             black_scholes(callResultNumpy, putResultNumpy, stockPrice,
                           optionStrike, optionYears, RISKFREE, VOLATILITY)
 
-
-
         @cuda.jit(argtypes=(double,), restype=double, device=True, inline=True)
         def cnd_cuda(d):
             K = 1.0 / (1.0 + 0.2316419 * math.fabs(d))
@@ -78,15 +73,15 @@ class TestBlackScholes(SerialMixin, unittest.TestCase):
                 ret_val = 1.0 - ret_val
             return ret_val
 
-
-        @cuda.jit(argtypes=(double[:], double[:], double[:], double[:], double[:],
-                            double, double))
+        @cuda.jit(argtypes=(double[:], double[:], double[:], double[:],
+                            double[:], double, double))
         def black_scholes_cuda(callResult, putResult, S, X, T, R, V):
             i = cuda.threadIdx.x + cuda.blockIdx.x * cuda.blockDim.x
             if i >= S.shape[0]:
                 return
             sqrtT = math.sqrt(T[i])
-            d1 = (math.log(S[i] / X[i]) + (R + 0.5 * V * V) * T[i]) / (V * sqrtT)
+            d1 = ((math.log(S[i] / X[i]) + (R + 0.5 * V * V) * T[i])
+                  / (V * sqrtT))
             d2 = d1 - V * sqrtT
             cndd1 = cnd_cuda(d1)
             cndd2 = cnd_cuda(d2)
@@ -96,7 +91,6 @@ class TestBlackScholes(SerialMixin, unittest.TestCase):
             putResult[i] = (X[i] * expRT * (1.0 - cndd2) - S[i] * (1.0 - cndd1))
 
         # numbapro
-        time0 = time.time()
         blockdim = 512, 1
         griddim = int(math.ceil(float(OPT_N) / blockdim[0])), 1
         stream = cuda.stream()
@@ -105,7 +99,7 @@ class TestBlackScholes(SerialMixin, unittest.TestCase):
         d_stockPrice = cuda.to_device(stockPrice, stream)
         d_optionStrike = cuda.to_device(optionStrike, stream)
         d_optionYears = cuda.to_device(optionYears, stream)
-        time1 = time.time()
+
         for i in range(iterations):
             black_scholes_cuda[griddim, blockdim, stream](
                 d_callResult, d_putResult, d_stockPrice, d_optionStrike,
@@ -114,8 +108,6 @@ class TestBlackScholes(SerialMixin, unittest.TestCase):
         d_putResult.copy_to_host(putResultNumbapro, stream)
         stream.synchronize()
 
-        dt = (time1 - time0)
-
         delta = np.abs(callResultNumpy - callResultNumbapro)
         L1norm = delta.sum() / np.abs(callResultNumpy).sum()
 
