@@ -1,8 +1,10 @@
-from numba import unittest_support as unittest
-import numpy as np
-from numba import cuda, types
 import struct
-from numba.cuda.testing import SerialMixin
+import numpy as np
+
+from numba import cuda
+from numba.core import types
+from numba.cuda.testing import CUDATestCase
+import unittest
 
 
 def float_to_int(x):
@@ -21,7 +23,7 @@ def float_to_complex(x):
     return np.complex128(x)
 
 
-class TestCasting(SerialMixin, unittest.TestCase):
+class TestCasting(CUDATestCase):
     def _create_wrapped(self, pyfunc, intype, outtype):
         wrapped_func = cuda.jit(device=True)(pyfunc)
 
@@ -33,7 +35,7 @@ class TestCasting(SerialMixin, unittest.TestCase):
             argarray = np.zeros(1, dtype=intype)
             argarray[0] = arg
             resarray = np.zeros(1, dtype=outtype)
-            cuda_wrapper_fn(argarray, resarray)
+            cuda_wrapper_fn[1, 1](argarray, resarray)
             return resarray[0]
 
         return wrapper_fn
@@ -59,8 +61,8 @@ class TestCasting(SerialMixin, unittest.TestCase):
         cfunc = self._create_wrapped(pyfunc, np.float32, np.uint32)
 
         self.assertEqual(cfunc(3.21), pyfunc(3.21))
-        self.assertEqual(cfunc(3.21), struct.unpack('I', struct.pack('i',
-                                                                      3))[0])
+        self.assertEqual(cfunc(3.21),
+                         struct.unpack('I', struct.pack('i', 3))[0])
 
     def test_float_to_complex(self):
         pyfunc = float_to_complex
