@@ -1,13 +1,9 @@
-from __future__ import print_function, absolute_import, division
-
-import numpy as np
-
-from numba import unittest_support as unittest
 from numba import cuda, float32
-from numba.cuda.testing import SerialMixin
+from numba.cuda.testing import CUDATestCase
+import unittest
 
 
-class TestFastMathOption(SerialMixin, unittest.TestCase):
+class TestFastMathOption(CUDATestCase):
     def test_kernel(self):
 
         def foo(arr, val):
