@@ -1,26 +1,26 @@
-from __future__ import absolute_import, print_function, division
-
 import sys
 import multiprocessing as mp
+import itertools
 import traceback
 import pickle
 
 import numpy as np
 
 from numba import cuda
-from numba.cuda.cudadrv import drvapi, devicearray
-from numba import unittest_support as unittest
-from numba.cuda.testing import skip_on_cudasim, CUDATestCase
+from numba.cuda.testing import (skip_on_cudasim, skip_under_cuda_memcheck,
+                                ContextResettingTestCase, ForeignArray)
+from numba.tests.support import linux_only
+import unittest
 
-
-not_linux = not sys.platform.startswith('linux')
+linux = sys.platform.startswith('linux')
 has_mp_get_context = hasattr(mp, 'get_context')
 
 
 def core_ipc_handle_test(the_work, result_queue):
     try:
         arr = the_work()
-    except:
+    # Catch anything going wrong in the worker function
+    except:  # noqa: E722
         # FAILED. propagate the exception as a string
         succ = False
         out = traceback.format_exc()
@@ -69,8 +69,8 @@ def ipc_array_test(ipcarr, result_queue):
                     raise AssertionError('invalid exception message')
             else:
                 raise AssertionError('did not raise on reopen')
-
-    except:
+    # Catch any exception so we can propagate it
+    except:  # noqa: E722
         # FAILED. propagate the exception as a string
         succ = False
         out = traceback.format_exc()
@@ -81,10 +81,11 @@ def ipc_array_test(ipcarr, result_queue):
     result_queue.put((succ, out))
 
 
-@unittest.skipIf(not_linux, "IPC only supported on Linux")
+@linux_only
+@skip_under_cuda_memcheck('Hangs cuda-memcheck')
 @unittest.skipUnless(has_mp_get_context, "requires multiprocessing.get_context")
 @skip_on_cudasim('Ipc not available in CUDASIM')
-class TestIpcMemory(CUDATestCase):
+class TestIpcMemory(ContextResettingTestCase):
     def test_ipc_handle(self):
         # prepare data for IPC
         arr = np.arange(10, dtype=np.intp)
@@ -111,12 +112,22 @@ class TestIpcMemory(CUDATestCase):
             np.testing.assert_equal(arr, out)
         proc.join(3)
 
-    def check_ipc_handle_serialization(self, index_arg=None):
+    def variants(self):
+        # Test with no slicing and various different slices
+        indices = (None, slice(3, None), slice(3, 8), slice(None, 8))
+        # Test with a Numba DeviceNDArray, or an array from elsewhere through
+        # the CUDA Array Interface
+        foreigns = (False, True)
+        return itertools.product(indices, foreigns)
+
+    def check_ipc_handle_serialization(self, index_arg=None, foreign=False):
         # prepare data for IPC
         arr = np.arange(10, dtype=np.intp)
         devarr = cuda.to_device(arr)
         if index_arg is not None:
             devarr = devarr[index_arg]
+        if foreign:
+            devarr = cuda.as_cuda_array(ForeignArray(devarr))
         expect = devarr.copy_to_host()
 
         # create IPC handle
@@ -144,20 +155,19 @@ class TestIpcMemory(CUDATestCase):
         proc.join(3)
 
     def test_ipc_handle_serialization(self):
-        # test no slicing
-        self.check_ipc_handle_serialization()
-        # slicing tests
-        self.check_ipc_handle_serialization(slice(3, None))
-        self.check_ipc_handle_serialization(slice(3, 8))
-        self.check_ipc_handle_serialization(slice(None, 8))
+        for index, foreign, in self.variants():
+            with self.subTest(index=index, foreign=foreign):
+                self.check_ipc_handle_serialization(index, foreign)
 
-    def check_ipc_array(self, index_arg=None):
+    def check_ipc_array(self, index_arg=None, foreign=False):
         # prepare data for IPC
         arr = np.arange(10, dtype=np.intp)
         devarr = cuda.to_device(arr)
         # Slice
         if index_arg is not None:
             devarr = devarr[index_arg]
+        if foreign:
+            devarr = cuda.as_cuda_array(ForeignArray(devarr))
         expect = devarr.copy_to_host()
         ipch = devarr.get_ipc_handle()
 
@@ -175,16 +185,14 @@ class TestIpcMemory(CUDATestCase):
         proc.join(3)
 
     def test_ipc_array(self):
-        # test no slicing
-        self.check_ipc_array()
-        # slicing tests
-        self.check_ipc_array(slice(3, None))
-        self.check_ipc_array(slice(3, 8))
-        self.check_ipc_array(slice(None, 8))
+        for index, foreign, in self.variants():
+            with self.subTest(index=index, foreign=foreign):
+                self.check_ipc_array(index, foreign)
 
-@unittest.skipUnless(not_linux, "Only on OS other than Linux")
+
+@unittest.skipIf(linux, 'Only on OS other than Linux')
 @skip_on_cudasim('Ipc not available in CUDASIM')
-class TestIpcNotSupported(CUDATestCase):
+class TestIpcNotSupported(ContextResettingTestCase):
     def test_unsupported(self):
         arr = np.arange(10, dtype=np.intp)
         devarr = cuda.to_device(arr)
@@ -198,14 +206,12 @@ def staged_ipc_handle_test(handle, device_num, result_queue):
     def the_work():
         with cuda.gpus[device_num]:
             this_ctx = cuda.devices.get_context()
-            can_access = handle.can_access_peer(this_ctx)
-            print('can_access_peer {} {}'.format(this_ctx, can_access))
             deviceptr = handle.open_staged(this_ctx)
             arrsize = handle.size // np.dtype(np.intp).itemsize
             hostarray = np.zeros(arrsize, dtype=np.intp)
             cuda.driver.device_to_host(
-                hostarray, deviceptr,  size=handle.size,
-                )
+                hostarray, deviceptr, size=handle.size,
+            )
             handle.close()
         return hostarray
 
@@ -215,8 +221,6 @@ def staged_ipc_handle_test(handle, device_num, result_queue):
 def staged_ipc_array_test(ipcarr, device_num, result_queue):
     try:
         with cuda.gpus[device_num]:
-            this_ctx = cuda.devices.get_context()
-            print(this_ctx.device)
             with ipcarr as darr:
                 arr = darr.copy_to_host()
                 try:
@@ -228,7 +232,8 @@ def staged_ipc_array_test(ipcarr, device_num, result_queue):
                         raise AssertionError('invalid exception message')
                 else:
                     raise AssertionError('did not raise on reopen')
-    except:
+    # Catch any exception so we can propagate it
+    except:  # noqa: E722
         # FAILED. propagate the exception as a string
         succ = False
         out = traceback.format_exc()
@@ -239,10 +244,11 @@ def staged_ipc_array_test(ipcarr, device_num, result_queue):
     result_queue.put((succ, out))
 
 
-@unittest.skipIf(not_linux, "IPC only supported on Linux")
+@linux_only
+@skip_under_cuda_memcheck('Hangs cuda-memcheck')
 @unittest.skipUnless(has_mp_get_context, "requires multiprocessing.get_context")
 @skip_on_cudasim('Ipc not available in CUDASIM')
-class TestIpcStaged(CUDATestCase):
+class TestIpcStaged(ContextResettingTestCase):
     def test_staged(self):
         # prepare data for IPC
         arr = np.arange(10, dtype=np.intp)
