@@ -1,9 +1,8 @@
-from __future__ import print_function, division, absolute_import
-
 import numpy as np
 
-from numba import cuda, config, float32
-from numba.cuda.testing import unittest, SerialMixin
+from numba import cuda, float32
+from numba.cuda.testing import unittest, CUDATestCase
+from numba.core import config
 
 # Ensure the test takes a reasonable amount of time in the simulator
 if config.ENABLE_CUDASIM:
@@ -15,7 +14,7 @@ n = bpg * tpb
 SM_SIZE = (tpb, tpb)
 
 
-class TestCudaMatMul(SerialMixin, unittest.TestCase):
+class TestCudaMatMul(CUDATestCase):
 
     def test_func(self):
 
