@@ -75,6 +75,24 @@ def _fill_ufunc_db(ufunc_db):
         'D->D': numbers.complex_negate_impl,
     }
 
+    ufunc_db[np.positive] = {
+        '?->?': numbers.int_positive_impl,
+        'b->b': numbers.int_positive_impl,
+        'B->B': numbers.int_positive_impl,
+        'h->h': numbers.int_positive_impl,
+        'H->H': numbers.int_positive_impl,
+        'i->i': numbers.int_positive_impl,
+        'I->I': numbers.int_positive_impl,
+        'l->l': numbers.int_positive_impl,
+        'L->L': numbers.int_positive_impl,
+        'q->q': numbers.int_positive_impl,
+        'Q->Q': numbers.int_positive_impl,
+        'f->f': numbers.real_positive_impl,
+        'd->d': numbers.real_positive_impl,
+        'F->F': numbers.complex_positive_impl,
+        'D->D': numbers.complex_positive_impl,
+    }
+
     ufunc_db[np.absolute] = {
         '?->?': numbers.int_abs_impl,
         'b->b': numbers.int_abs_impl,
@@ -231,6 +249,21 @@ def _fill_ufunc_db(ufunc_db):
         'dd->d': npyfuncs.np_real_mod_impl,
     }
 
+    ufunc_db[np.divmod] = {
+        'bb->bb': npyfuncs.np_int_sdivrem_impl,
+        'BB->BB': npyfuncs.np_int_udivrem_impl,
+        'hh->hh': npyfuncs.np_int_sdivrem_impl,
+        'HH->HH': npyfuncs.np_int_udivrem_impl,
+        'ii->ii': npyfuncs.np_int_sdivrem_impl,
+        'II->II': npyfuncs.np_int_udivrem_impl,
+        'll->ll': npyfuncs.np_int_sdivrem_impl,
+        'LL->LL': npyfuncs.np_int_udivrem_impl,
+        'qq->qq': npyfuncs.np_int_sdivrem_impl,
+        'QQ->QQ': npyfuncs.np_int_udivrem_impl,
+        'ff->ff': npyfuncs.np_real_divmod_impl,
+        'dd->dd': npyfuncs.np_real_divmod_impl,
+    }
+
     ufunc_db[np.fmod] = {
         'bb->b': npyfuncs.np_int_fmod_impl,
         'BB->B': npyfuncs.np_int_fmod_impl,
@@ -1010,6 +1043,9 @@ def _fill_ufunc_db(ufunc_db):
     ufunc_db[np.negative].update({
         'm->m': npdatetime.timedelta_neg_impl,
     })
+    ufunc_db[np.positive].update({
+        'm->m': npdatetime.timedelta_pos_impl,
+    })
     ufunc_db[np.absolute].update({
         'm->m': npdatetime.timedelta_abs_impl,
     })
