@@ -1,8 +1,6 @@
-from __future__ import print_function, absolute_import
-
 import numpy as np
 
-import numba.unittest_support as unittest
+import unittest
 from numba import roc
 
 
