@@ -1,15 +1,14 @@
-from __future__ import print_function, division, absolute_import
-
 import gc
+from io import StringIO
 
 import numpy as np
 
 from numba import njit, vectorize
-from numba import unittest_support as unittest
-from numba import compiler, typing, typeof, ir, utils, types
-from numba.compiler import Compiler, Flags
-from numba.targets import cpu
-from .support import MemoryLeakMixin, TestCase
+from numba import typeof
+from numba.core import utils, types, typing, ir, compiler, cpu
+from numba.core.compiler import Compiler, Flags
+from numba.tests.support import MemoryLeakMixin, TestCase
+import unittest
 
 
 class Namespace(dict):
@@ -462,12 +461,27 @@ class TestRewriteIssues(MemoryLeakMixin, TestCase):
         a = np.linspace(0, 1, 10)
         cfunc(a, a, a, a)
 
-        buf = utils.StringIO()
+        buf = StringIO()
         cfunc.inspect_types(buf)
         res = buf.getvalue()
         self.assertIn("#   u.1 = ", res)
         self.assertIn("#   u.2 = ", res)
 
+    def test_issue_5599_name_collision(self):
+        # The original error will fail in lowering of the array_expr
+        @njit
+        def f(x):
+            arr = np.ones(x)
+
+            for _ in range(2):
+                val =  arr * arr
+                arr = arr.copy()
+            return arr
+
+        got = f(5)
+        expect = f.py_func(5)
+        np.testing.assert_array_equal(got, expect)
+
 
 class TestSemantics(MemoryLeakMixin, unittest.TestCase):
 
@@ -517,7 +531,6 @@ class TestOptionals(MemoryLeakMixin, unittest.TestCase):
         self.assertTrue(isinstance(oty, types.Optional))
         self.assertTrue(isinstance(oty.type, types.Float))
 
-
     def test_optional_array_type(self):
 
         @njit
