@@ -1,18 +1,17 @@
-from __future__ import print_function, absolute_import, division
-
+from ctypes import *
 import sys
 import threading
 
 import numpy as np
 
-from numba.ctypes_support import *
 
-from numba import unittest_support as unittest
-from numba.compiler import compile_isolated
-from numba import jit, types, errors
-from numba.typing import ctypes_utils
-from .support import MemoryLeakMixin, tag, TestCase
-from .ctypes_usecases import *
+from numba.core.compiler import compile_isolated
+from numba import jit
+from numba.core import types, errors
+from numba.core.typing import ctypes_utils
+from numba.tests.support import MemoryLeakMixin, tag, TestCase
+from numba.tests.ctypes_usecases import *
+import unittest
 
 
 class TestCTypesTypes(TestCase):
@@ -31,7 +30,6 @@ class TestCTypesTypes(TestCase):
 
         check(None, types.void)
 
-    @tag('important')
     def test_from_ctypes(self):
         """
         Test converting a ctypes type to a Numba type.
@@ -47,7 +45,6 @@ class TestCTypesTypes(TestCase):
             ctypes_utils.from_ctypes(c_wchar_p)
         self.assertIn("Unsupported ctypes type", str(raises.exception))
 
-    @tag('important')
     def test_to_ctypes(self):
         """
         Test converting a Numba type to a ctypes type.
@@ -67,7 +64,6 @@ class TestCTypesTypes(TestCase):
 
 class TestCTypesUseCases(MemoryLeakMixin, TestCase):
 
-    @tag('important')
     def test_c_sin(self):
         pyfunc = use_c_sin
         cres = compile_isolated(pyfunc, [types.double])
@@ -181,7 +177,6 @@ class TestCTypesUseCases(MemoryLeakMixin, TestCase):
         for got in outputs:
             self.assertEqual(expected, got)
 
-    @tag('important')
     def test_passing_array_ctypes_data(self):
         """
         Test the ".ctypes.data" attribute of an array can be passed
@@ -208,7 +203,6 @@ class TestCTypesUseCases(MemoryLeakMixin, TestCase):
         self.assertPreciseEqual(expected, got)
         return cfunc
 
-    @tag('important')
     def test_passing_array_ctypes_voidptr(self):
         """
         Test the ".ctypes" attribute of an array can be passed
@@ -216,7 +210,6 @@ class TestCTypesUseCases(MemoryLeakMixin, TestCase):
         """
         self.check_array_ctypes(use_c_vsquare)
 
-    @tag('important')
     def test_passing_array_ctypes_voidptr_pass_ptr(self):
         """
         Test the ".ctypes" attribute of an array can be passed
@@ -227,7 +220,8 @@ class TestCTypesUseCases(MemoryLeakMixin, TestCase):
         # Non-compatible pointers are not accepted (here float32* vs. float64*)
         with self.assertRaises(errors.TypingError) as raises:
             cfunc(np.float32([0.0]))
-        self.assertIn("Invalid use of ExternalFunctionPointer",
+
+        self.assertIn("No implementation of function ExternalFunctionPointer",
                       str(raises.exception))
 
     def test_storing_voidptr_to_int_array(self):
