@@ -1,7 +1,6 @@
 """
 Tests gdb bindings
 """
-from __future__ import print_function
 import os
 import platform
 import subprocess
@@ -9,13 +8,15 @@ import sys
 import threading
 from itertools import permutations
 
-from numba import njit, gdb, gdb_init, gdb_breakpoint, prange, errors
+from numba import njit, gdb, gdb_init, gdb_breakpoint, prange
+from numba.core import errors
 from numba import jit
-from numba import unittest_support as unittest
-from numba.targets.gdb_hook import _confirm_gdb
+from numba.misc.gdb_hook import _confirm_gdb
+
+from numba.tests.support import (TestCase, captured_stdout, tag,
+                                 skip_parfors_unsupported)
+import unittest
 
-from .support import (TestCase, captured_stdout, tag)
-from .test_parfors import skip_unsupported as parfors_skip_unsupported
 
 _platform = sys.platform
 
@@ -119,19 +120,19 @@ class TestGdbBindImpls(TestCase):
         with captured_stdout():
             _dbg_jit(impl_gdb_call_w_bp)(10)
 
-    @parfors_skip_unsupported
+    @skip_parfors_unsupported
     @needs_gdb_harness
     def test_gdb_split_init_and_break_w_parallel_cpython_impl(self):
         with captured_stdout():
             impl_gdb_split_init_and_break_w_parallel(10)
 
-    @parfors_skip_unsupported
+    @skip_parfors_unsupported
     @needs_gdb_harness
     def test_gdb_split_init_and_break_w_parallel_nopython_impl(self):
         with captured_stdout():
             _dbg_njit(impl_gdb_split_init_and_break_w_parallel)(10)
 
-    @parfors_skip_unsupported
+    @skip_parfors_unsupported
     @needs_gdb_harness
     def test_gdb_split_init_and_break_w_parallel_objmode_impl(self):
         with captured_stdout():
