@@ -6,7 +6,6 @@ from numba.core.compiler import compile_isolated, Flags
 from numba import jit, njit
 from numba.core import types
 from numba.tests.support import TestCase, MemoryLeakMixin
-from numba import testing
 from numba.core.datamodel.testing import test_factory
 
 
@@ -342,15 +341,6 @@ class TestGenerators(MemoryLeakMixin, TestCase):
         self.assertIn(msg % types.intp.bitwidth, str(e.exception))
 
 
-class TestGenExprs(MemoryLeakMixin, TestCase):
-    @testing.allow_interpreter_mode
-    def test_return_generator_expr(self):
-        pyfunc = return_generator_expr
-        cr = compile_isolated(pyfunc, ())
-        cfunc = cr.entry_point
-        self.assertEqual(sum(cfunc([1, 2, 3])), sum(pyfunc([1, 2, 3])))
-
-
 def nrt_gen0(ary):
     for elem in ary:
         yield elem
