@@ -1,12 +1,11 @@
 # -*- coding: utf-8 -*-
-from __future__ import print_function, absolute_import
 
 import re
 
-from numba import unittest_support as unittest
-from numba import itanium_mangler
 from numba import int32, int64, uint32, uint64, float32, float64
-from numba.types import range_iter32_type
+from numba.core.types import range_iter32_type
+from numba.core import itanium_mangler
+import unittest
 
 
 class TestItaniumManager(unittest.TestCase):
