@@ -1,12 +1,11 @@
-from __future__ import print_function
-
 import numpy as np
 
 from numba import njit
-import numba.unittest_support as unittest
-from numba.compiler import compile_isolated, Flags
-from numba import numpy_support, types, errors
-from .support import TestCase, MemoryLeakMixin, tag
+import unittest
+from numba.core.compiler import compile_isolated, Flags
+from numba.core import types, errors
+from numba.tests.support import TestCase, MemoryLeakMixin, tag
+from numba.np import numpy_support
 
 enable_pyobj_flags = Flags()
 enable_pyobj_flags.set("enable_pyobject")
@@ -96,7 +95,6 @@ class IterationTest(MemoryLeakMixin, TestCase):
     def test_int_tuple_iter(self, flags=force_pyobj_flags):
         self.run_nullary_func(int_tuple_iter_usecase, flags)
 
-    @tag('important')
     def test_int_tuple_iter_npm(self):
         self.test_int_tuple_iter(flags=no_pyobj_flags)
 
@@ -112,21 +110,18 @@ class IterationTest(MemoryLeakMixin, TestCase):
     def test_tuple_tuple_iter(self, flags=force_pyobj_flags):
         self.run_nullary_func(tuple_tuple_iter_usecase, flags)
 
-    @tag('important')
     def test_tuple_tuple_iter_npm(self):
         self.test_tuple_tuple_iter(flags=no_pyobj_flags)
 
     def test_enumerate_nested_tuple(self, flags=force_pyobj_flags):
         self.run_nullary_func(enumerate_nested_tuple_usecase, flags)
 
-    @tag('important')
     def test_enumerate_nested_tuple_npm(self):
         self.test_enumerate_nested_tuple(flags=no_pyobj_flags)
 
     def test_nested_enumerate(self, flags=force_pyobj_flags):
         self.run_nullary_func(nested_enumerate_usecase, flags)
 
-    @tag('important')
     def test_nested_enumerate_npm(self):
         self.test_nested_enumerate(flags=no_pyobj_flags)
 
@@ -155,7 +150,6 @@ class IterationTest(MemoryLeakMixin, TestCase):
     def test_array_1d_complex(self, flags=force_pyobj_flags):
         self.run_array_1d(types.complex128, np.arange(5.0) * 1.0j, flags)
 
-    @tag('important')
     def test_array_1d_complex_npm(self):
         self.test_array_1d_complex(no_pyobj_flags)
 
@@ -194,23 +188,18 @@ class IterationTest(MemoryLeakMixin, TestCase):
     def test_array_1d_record_mutate(self):
         self.test_array_1d_record_mutate_npm(flags=force_pyobj_flags)
 
-    def test_array_2d_raises(self):
+    def test_array_0d_raises(self):
 
         def foo(x):
             for i in x:
                 pass
 
-        # 1D is fine
-        aryty = types.Array(types.int32, 1, 'C')
-        compile_isolated(foo, (aryty,))
-
-        # 2d is typing error
+        # 0d is typing error
         with self.assertRaises(errors.TypingError) as raises:
-            aryty = types.Array(types.int32, 2, 'C')
+            aryty = types.Array(types.int32, 0, 'C')
             compile_isolated(foo, (aryty,))
 
-        self.assertIn("Direct iteration is not supported",
-                      str(raises.exception))
+        self.assertIn("0-d array", str(raises.exception))
 
     def test_tuple_iter_issue1504(self):
         # The issue is due to `row` being typed as heterogeneous tuple.
