@@ -1,5 +1,3 @@
-from __future__ import print_function
-
 from collections import namedtuple
 import contextlib
 import itertools
@@ -8,11 +6,14 @@ import sys
 import ctypes as ct
 import numpy as np
 
-from numba.compiler import compile_isolated, Flags
-from numba import jit, types, utils, typeof, jitclass
-import numba.unittest_support as unittest
-from numba import testing, errors
-from .support import TestCase, MemoryLeakMixin, tag
+from numba.core.compiler import compile_isolated, Flags
+from numba import jit, typeof, njit, literal_unroll, literally
+import unittest
+from numba import testing
+from numba.core import types, utils, errors
+from numba.tests.support import TestCase, MemoryLeakMixin, tag
+from numba.experimental import jitclass
+from numba.core.extending import overload
 
 
 enable_pyobj_flags = Flags()
@@ -394,21 +395,6 @@ class TestLists(MemoryLeakMixin, TestCase):
         cfunc = cr.entry_point
         self.assertEqual(cfunc(1, 2, 3, 4, 5, 6), pyfunc(1, 2, 3, 4, 5, 6))
 
-    @testing.allow_interpreter_mode
-    def test_list_comprehension(self):
-        list_tests = [list_comprehension1,
-                      list_comprehension2,
-                      list_comprehension3,
-                      list_comprehension4,
-                      list_comprehension5,
-                      list_comprehension6]
-
-        for test in list_tests:
-            pyfunc = test
-            cr = compile_isolated(pyfunc, ())
-            cfunc = cr.entry_point
-            self.assertEqual(cfunc(), pyfunc())
-
     def check_unary_with_size(self, pyfunc, precise=True):
         cfunc = jit(nopython=True)(pyfunc)
         # Use various sizes, to stress the allocation algorithm
@@ -442,21 +428,18 @@ class TestLists(MemoryLeakMixin, TestCase):
     def test_append(self):
         self.check_unary_with_size(list_append)
 
-    @tag('important')
     def test_append_heterogeneous(self):
         self.check_unary_with_size(list_append_heterogeneous, precise=False)
 
     def test_extend(self):
         self.check_unary_with_size(list_extend)
 
-    @tag('important')
     def test_extend_heterogeneous(self):
         self.check_unary_with_size(list_extend_heterogeneous, precise=False)
 
     def test_pop0(self):
         self.check_unary_with_size(list_pop0)
 
-    @tag('important')
     def test_pop1(self):
         pyfunc = list_pop1
         cfunc = jit(nopython=True)(pyfunc)
@@ -488,11 +471,9 @@ class TestLists(MemoryLeakMixin, TestCase):
     def test_len(self):
         self.check_unary_with_size(list_len)
 
-    @tag('important')
     def test_getitem(self):
         self.check_unary_with_size(list_getitem)
 
-    @tag('important')
     def test_setitem(self):
         self.check_unary_with_size(list_setitem)
 
@@ -518,7 +499,6 @@ class TestLists(MemoryLeakMixin, TestCase):
                 expected = pyfunc(n, n_src, start, stop)
                 self.assertPreciseEqual(cfunc(n, n_src, start, stop), expected)
 
-    @tag('important')
     def test_getslice3(self):
         pyfunc = list_getslice3
         cfunc = jit(nopython=True)(pyfunc)
@@ -529,7 +509,6 @@ class TestLists(MemoryLeakMixin, TestCase):
                 expected = pyfunc(n, start, stop, step)
                 self.assertPreciseEqual(cfunc(n, start, stop, step), expected)
 
-    @tag('important')
     def test_setslice3(self):
         pyfunc = list_setslice3
         cfunc = jit(nopython=True)(pyfunc)
@@ -558,7 +537,6 @@ class TestLists(MemoryLeakMixin, TestCase):
     def test_delslice1(self):
         self.check_slicing2(list_delslice1)
 
-    @tag('important')
     def test_delslice2(self):
         self.check_slicing2(list_delslice2)
 
@@ -573,7 +551,6 @@ class TestLists(MemoryLeakMixin, TestCase):
     def test_iteration(self):
         self.check_unary_with_size(list_iteration)
 
-    @tag('important')
     def test_reverse(self):
         self.check_unary_with_size(list_reverse)
 
@@ -638,13 +615,9 @@ class TestLists(MemoryLeakMixin, TestCase):
         for v in range(5):
             self.assertPreciseEqual(cfunc(18, v), pyfunc(18, v))
 
-    @unittest.skipUnless(sys.version_info >= (3, 3),
-                         "list.clear() needs Python 3.3+")
     def test_clear(self):
         self.check_unary_with_size(list_clear)
 
-    @unittest.skipUnless(sys.version_info >= (3, 3),
-                         "list.copy() needs Python 3.3+")
     def test_copy(self):
         self.check_unary_with_size(list_copy)
 
@@ -798,7 +771,6 @@ class TestUnboxing(MemoryLeakMixin, TestCase):
         check([(1, 2j), (3, 4j)])
         check([(), (), ()])
 
-    @tag('important')
     def test_list_inside_tuple(self):
         check = self.check_unary(unbox_usecase3)
         check((1, [2, 3, 4]))
@@ -830,15 +802,10 @@ class TestUnboxing(MemoryLeakMixin, TestCase):
         lst = [(1,), (2, 3)]
         with self.assertRaises(TypeError) as raises:
             cfunc(lst)
-        if utils.IS_PY3:
-            msg = ("can't unbox heterogeneous list: "
-                   "UniTuple({0} x 1) != UniTuple({0} x 2)")
-            self.assertEqual(str(raises.exception), msg.format(types.intp))
-        else:
-            self.assertEqual(
-                str(raises.exception),
-                "can't unbox heterogeneous list",
-                )
+        msg = ("can't unbox heterogeneous list: "
+                "UniTuple({0} x 1) != UniTuple({0} x 2)")
+        self.assertEqual(str(raises.exception), msg.format(types.intp))
+
 
 
 class TestListReflection(MemoryLeakMixin, TestCase):
@@ -881,7 +848,6 @@ class TestListReflection(MemoryLeakMixin, TestCase):
                 cfunc(l)
             self.assertPreciseEqual(l, [1, 2, 3, 42])
 
-    @tag('important')
     def test_reflect_same_list(self):
         """
         When the same list object is reflected twice, behaviour should
@@ -1277,7 +1243,6 @@ class TestListOfList(ManagedListTestCase):
             str(raises.exception),
             )
 
-    @unittest.skipUnless(utils.IS_PY3, "Py3 only due to ordering of error")
     @expect_reflection_failure
     def test_c05(self):
         def bar(x):
@@ -1288,7 +1253,6 @@ class TestListOfList(ManagedListTestCase):
         r = [[np.arange(3).astype(np.intp)]]
         self.compile_and_test(bar, r)
 
-    @unittest.skipUnless(utils.IS_PY3, "Py3 only due to ordering of error")
     def test_c06(self):
         def bar(x):
             f = x
@@ -1434,5 +1398,419 @@ class TestListAndJitClasses(ManagedListTestCase):
         self.assert_list_element_precise_equal(got.data, expect.data)
 
 
+class TestListInitialValues(MemoryLeakMixin, TestCase):
+    """Tests that lists carry their initial value if present"""
+
+    def test_homogeneous_and_literal(self):
+        def bar(l):
+            ...
+
+        @overload(bar)
+        def ol_bar(l):
+            if l.initial_value is None:
+                return lambda l: literally(l)
+            self.assertTrue(isinstance(l, types.List))
+            self.assertEqual(l.initial_value, [1, 2, 3])
+            self.assertEqual(hasattr(l, 'literal_value'), False)
+            return lambda l: l
+
+        @njit
+        def foo():
+            # keys and values all have literal representation
+            x = [1, 2, 3]
+            bar(x)
+
+        foo()
+
+    def test_heterogeneous_but_castable_to_homogeneous(self):
+        def bar(l):
+            ...
+
+        @overload(bar)
+        def ol_bar(l):
+            self.assertTrue(isinstance(l, types.List))
+            self.assertEqual(l.initial_value, None)
+            self.assertEqual(hasattr(l, 'literal_value'), False)
+            return lambda l: l
+
+        @njit
+        def foo():
+            # This list will be typed based on 1j, i.e. complex128
+            # as the values are not all literals, there's no "initial_value"
+            # available irrespective of whether it's possible to rip this
+            # information out of the bytecode.
+            x = [1j, 2, 3]
+            bar(x)
+
+        foo()
+
+    def test_mutation_not_carried(self):
+        def bar(d):
+            ...
+
+        @overload(bar)
+        def ol_bar(d):
+            if d.initial_value is None:
+                return lambda d: literally(d)
+            self.assertTrue(isinstance(d, types.List))
+            self.assertEqual(d.initial_value, [1, 2, 3])
+            return lambda d: d
+
+        @njit
+        def foo():
+            # This list is mutated, check the initial_value carries
+            # correctly and is not mutated
+            x = [1, 2, 3]
+            x.append(4)
+            bar(x)
+
+        foo()
+
+    def test_mutation_not_carried_single_function(self):
+        # this is another pattern for using literally
+
+        @njit
+        def nop(*args):
+            pass
+
+        for fn, iv in (nop, None), (literally, [1, 2, 3]):
+            @njit
+            def baz(x):
+                pass
+
+            def bar(z):
+                pass
+
+            @overload(bar)
+            def ol_bar(z):
+                def impl(z):
+                    fn(z)
+                    baz(z)
+                return impl
+
+            @njit
+            def foo():
+                x = [1, 2, 3]
+                bar(x)
+                x.append(2)
+                return x
+
+            foo()
+            # baz should be specialised based on literally being invoked and
+            # the literal/unliteral arriving at the call site
+            larg = baz.signatures[0][0]
+            self.assertEqual(larg.initial_value, iv)
+
+    def test_list_of_list_ctor(self):
+        # see issue 6082
+        @njit
+        def bar(x):
+            pass
+
+        @njit
+        def foo():
+            x = [[1, 2, 3, 4, 5], [1, 2, 3, 4, 6]]
+            bar(x)
+
+        foo()
+        larg = bar.signatures[0][0]
+        self.assertEqual(larg.initial_value, None)
+        self.assertEqual(larg.dtype.initial_value, None)
+
+
+class TestLiteralLists(MemoryLeakMixin, TestCase):
+
+    def test_basic_compile(self):
+        @njit
+        def foo():
+            l = [1, 'a']
+
+        foo()
+
+    def test_literal_value_passthrough(self):
+
+        def bar(x):
+            pass
+
+        @overload(bar)
+        def ol_bar(x):
+            self.assertTrue(isinstance(x, types.LiteralList))
+            lv = x.literal_value
+            self.assertTrue(isinstance(lv, list))
+            self.assertEqual(lv[0], types.literal(1))
+            self.assertEqual(lv[1], types.literal('a'))
+            self.assertEqual(lv[2], types.Array(types.float64, 1, 'C'))
+            self.assertEqual(lv[3], types.List(types.intp, reflected=False,
+                                               initial_value=[1, 2, 3]))
+            self.assertTrue(isinstance(lv[4], types.LiteralList))
+            self.assertEqual(lv[4].literal_value[0], types.literal('cat'))
+            self.assertEqual(lv[4].literal_value[1], types.literal(10))
+            return lambda x: x
+
+        @njit
+        def foo():
+            otherhomogeneouslist = [1, 2, 3]
+            otherheterogeneouslist = ['cat', 10]
+            zeros = np.zeros(5)
+            l = [1, 'a', zeros, otherhomogeneouslist, otherheterogeneouslist]
+            bar(l)
+
+        foo()
+
+    def test_literal_value_involved_passthrough(self):
+
+        def bar(x):
+            pass
+
+        @overload(bar)
+        def ol_bar(x):
+            self.assertTrue(isinstance(x, types.LiteralStrKeyDict))
+            dlv = x.literal_value
+            inner_literal = {types.literal('g'): types.literal('h'),
+                             types.literal('i'):
+                                 types.Array(types.float64, 1, 'C')}
+            inner_dict = types.LiteralStrKeyDict(inner_literal)
+            outer_literal = {types.literal('a'):
+                            types.LiteralList([types.literal(1),
+                                               types.literal('a'),
+                                               types.DictType(
+                                                   types.unicode_type,
+                                                   types.intp,
+                                                   initial_value={'f': 1}),
+                                               inner_dict]),
+                        types.literal('b'): types.literal(2),
+                        types.literal('c'): types.List(types.complex128,
+                                                       reflected=False)}
+            def check_same(a, b):
+                if (isinstance(a, types.LiteralList) and
+                    isinstance(b, types.LiteralList)):
+                    for i, j in zip(a.literal_value, b.literal_value):
+                        check_same(a.literal_value, b.literal_value)
+                elif (isinstance(a, list) and
+                     isinstance(b, list)):
+                    for i, j in zip(a, b):
+                        check_same(i, j)
+                elif (isinstance(a, types.LiteralStrKeyDict) and
+                      isinstance(b, types.LiteralStrKeyDict)):
+                    for (ki, vi), (kj, vj) in zip(a.literal_value.items(),
+                                                  b.literal_value.items()):
+                        check_same(ki, kj)
+                        check_same(vi, vj)
+                elif (isinstance(a, dict) and
+                      isinstance(b, dict)):
+                    for (ki, vi), (kj, vj) in zip(a.items(), b.items()):
+                        check_same(ki, kj)
+                        check_same(vi, vj)
+                else:
+                    self.assertEqual(a, b)
+            check_same(dlv, outer_literal)
+            return lambda x: x
+
+        @njit
+        def foo():
+            # this stretches what's possible with LiteralStrKeyDict and
+            # LiteralList, it's nested and contains typed.Dict and np arrays
+            # as well as more constant type things.
+            l = {'a': [1, 'a', {'f': 1}, {'g': 'h', 'i': np.zeros(5)}], 'b': 2,
+                 'c': [1j, 2j, 3j]}
+            bar(l)
+
+        foo()
+
+    def test_mutation_failure(self):
+
+        def staticsetitem():
+            l = ['a', 1]
+            l[0] = 'b'
+
+        def delitem():
+            l = ['a', 1]
+            del l[0]
+
+        def append():
+            l = ['a', 1]
+            l.append(2j)
+
+        def extend():
+            l = ['a', 1]
+            l.extend([2j, 3j])
+
+        def insert():
+            l = ['a', 1]
+            l.insert(0, 2j)
+
+        def remove():
+            l = ['a', 1]
+            l.remove('a')
+
+        def pop():
+            l = ['a', 1]
+            l.pop()
+
+        def clear():
+            l = ['a', 1]
+            l.clear()
+
+        def sort():
+            l = ['a', 1]
+            l.sort()
+
+        def reverse():
+            l = ['a', 1]
+            l.reverse()
+
+        illegals = (staticsetitem, delitem, append, extend, insert, remove, pop,
+                    clear, sort, reverse)
+
+        for test in illegals:
+            with self.subTest(test.__name__):
+                with self.assertRaises(errors.TypingError) as raises:
+                    njit(test)()
+                expect = "Cannot mutate a literal list"
+                self.assertIn(expect, str(raises.exception))
+
+    def test_count(self):
+        @njit
+        def foo():
+            l = ['a', 1, 'a', 2, 'a', 3, 'b', 4, 'b', 5, 'c']
+            r = []
+            for x in 'abc':
+                r.append(l.count(x))
+            return r
+
+        self.assertEqual(foo.py_func(), foo())
+
+    def test_len(self):
+        @njit
+        def foo():
+            l = ['a', 1, 'a', 2, 'a', 3, 'b', 4, 'b', 5, 'c']
+            return len(l)
+
+        self.assertEqual(foo.py_func(), foo())
+
+    def test_contains(self):
+        @njit
+        def foo():
+            l = ['a', 1, 'a', 2, 'a', 3, 'b', 4, 'b', 5, 'c']
+            r = []
+            for x in literal_unroll(('a', 'd', 2, 6)):
+                r.append(x in l)
+            return r
+
+        self.assertEqual(foo.py_func(), foo())
+
+    def test_getitem(self):
+
+        @njit
+        def foo(x):
+            l = ['a', 1]
+            return l[x]
+
+        with self.assertRaises(errors.TypingError) as raises:
+            foo(0)
+        expect = "Cannot __getitem__ on a literal list"
+        self.assertIn(expect, str(raises.exception))
+
+    def test_staticgetitem(self):
+
+        @njit
+        def foo():
+            l = ['a', 1]
+            return l[0], l[1]
+
+        self.assertEqual(foo.py_func(), foo())
+
+    def test_staticgetitem_slice(self):
+        # this is forbidden by typing as there's no way to serialize a list of
+        # any kind as required by returning a (static) slice of a LiteralList
+        @njit
+        def foo():
+            l = ['a', 'b', 1]
+            return l[:2]
+
+        with self.assertRaises(errors.TypingError) as raises:
+            foo()
+        expect = "Cannot __getitem__ on a literal list"
+        self.assertIn(expect, str(raises.exception))
+
+    def test_setitem(self):
+
+        @njit
+        def foo(x):
+            l = ['a', 1]
+            l[x] = 'b'
+
+        with self.assertRaises(errors.TypingError) as raises:
+            foo(0)
+        expect = "Cannot mutate a literal list"
+        self.assertIn(expect, str(raises.exception))
+
+    def test_unify(self):
+
+        @njit
+        def foo(x):
+            if x + 1 > 3:
+                l = ['a', 1]
+            else:
+                l = ['b', 2]
+            return l[0]
+
+        for x in (-100, 100):
+            self.assertEqual(foo.py_func(x), foo(x))
+
+    def test_not_unify(self):
+
+        @njit
+        def foo(x):
+            if x + 1 > 3:
+                l = ['a', 1, 2j]
+            else:
+                l = ['b', 2]
+            return l[0]
+
+        with self.assertRaises(errors.TypingError) as raises:
+            foo(100)
+        expect = "Cannot unify LiteralList"
+        self.assertIn(expect, str(raises.exception))
+
+    def test_index(self):
+        @njit
+        def foo():
+            l = ['a', 1]
+            l.index('a')
+
+        with self.assertRaises(errors.TypingError) as raises:
+            foo()
+        expect = "list.index is unsupported for literal lists"
+        self.assertIn(expect, str(raises.exception))
+
+    def test_copy(self):
+        @njit
+        def foo():
+            l = ['a', 1].copy()
+            return l[0], l[1]
+
+        self.assertEqual(foo(), foo.py_func())
+
+    def test_tuple_not_in_mro(self):
+        # Related to #6094, make sure that LiteralList does not inherit from
+        # types.BaseTuple as this breaks isinstance checks.
+        def bar(x):
+            pass
+
+        @overload(bar)
+        def ol_bar(x):
+            self.assertFalse(isinstance(x, types.BaseTuple))
+            self.assertTrue(isinstance(x, types.LiteralList))
+            return lambda x: ...
+
+        @njit
+        def foo():
+            l = ['a', 1]
+            bar(l)
+
+        foo()
+
+
 if __name__ == '__main__':
     unittest.main()
