@@ -1,7 +1,6 @@
-from __future__ import print_function, absolute_import, division
-from numba import unittest_support as unittest
-from numba.compiler import compile_isolated
-from numba import types
+from numba.core.compiler import compile_isolated
+from numba.core import types
+import unittest
 
 
 def domax3(a, b, c):
