@@ -1,7 +1,6 @@
-from __future__ import print_function
-import numba.unittest_support as unittest
-from numba.compiler import compile_isolated, Flags
-from numba import types
+import unittest
+from numba.core.compiler import compile_isolated, Flags
+from numba.core import types
 
 enable_pyobj_flags = Flags()
 enable_pyobj_flags.set("enable_pyobject")
