@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 import math
 import os
 import platform
@@ -8,24 +6,27 @@ import re
 
 import numpy as np
 
-from numba import unittest_support as unittest
-from numba import njit, targets, typing, types
-from numba.compiler import compile_isolated, Flags
-from numba.runtime import (
+from numba import njit
+from numba.core import typing, types
+from numba.core.compiler import compile_isolated, Flags
+from numba.core.runtime import (
     rtsys,
     nrtopt,
     _nrt_python,
     nrt,
 )
-from numba.extending import intrinsic, include_path
-from numba.typing import signature
-from numba.targets.imputils import impl_ret_untracked
+from numba.core.extending import intrinsic, include_path
+from numba.core.typing import signature
+from numba.core.imputils import impl_ret_untracked
 from llvmlite import ir
 import llvmlite.binding as llvm
-from numba import cffi_support
-from numba.unsafe.nrt import NRT_get_api
+import numba.core.typing.cffi_utils as cffi_support
+from numba.core.unsafe.nrt import NRT_get_api
 
-from .support import MemoryLeakMixin, TestCase, temp_directory, import_dynamic
+from numba.tests.support import (MemoryLeakMixin, TestCase, temp_directory,
+                                 import_dynamic)
+from numba.core import cpu
+import unittest
 
 enable_nrt_flags = Flags()
 enable_nrt_flags.set("nrt")
@@ -81,7 +82,7 @@ class TestNrtMemInfo(unittest.TestCase):
         # Reset the Dummy class
         Dummy.alive = 0
         # initialize the NRT (in case the tests are run in isolation)
-        targets.cpu.CPUContext(typing.Context())
+        cpu.CPUContext(typing.Context())
 
     def test_meminfo_refct_1(self):
         d = Dummy()
@@ -119,7 +120,6 @@ class TestNrtMemInfo(unittest.TestCase):
         del mi
         self.assertEqual(Dummy.alive, 0)
 
-    @unittest.skipIf(sys.version_info < (3,), "memoryview not supported")
     def test_fake_memoryview(self):
         d = Dummy()
         self.assertEqual(Dummy.alive, 1)
@@ -142,7 +142,6 @@ class TestNrtMemInfo(unittest.TestCase):
         del mview
         self.assertEqual(Dummy.alive, 0)
 
-    @unittest.skipIf(sys.version_info < (3,), "memoryview not supported")
     def test_memoryview(self):
         from ctypes import c_uint32, c_void_p, POINTER, cast
 
@@ -221,8 +220,6 @@ class TestNrtMemInfo(unittest.TestCase):
         # consumed by another thread.
 
 
-@unittest.skipUnless(sys.version_info >= (3, 4),
-                     "need Python 3.4+ for the tracemalloc module")
 class TestTracemalloc(unittest.TestCase):
     """
     Test NRT-allocated memory can be tracked by tracemalloc.
@@ -485,6 +482,7 @@ br i1 %.294, label %B42, label %B160
         # no other lines
         self.assertEqual(len(list(pruned_lines.splitlines())), len(combined))
 
+    @unittest.skip("Pass removed as it was buggy. Re-enable when fixed.")
     def test_refct_pruning_with_branches(self):
         '''testcase from #2350'''
         @njit
@@ -585,7 +583,7 @@ class TestNrtExternalCFFI(MemoryLeakMixin, TestCase):
         name = "{}_test_manage_memory".format(self.__class__.__name__)
         source = r"""
 #include <stdio.h>
-#include "numba/runtime/nrt_external.h"
+#include "numba/core/runtime/nrt_external.h"
 
 int status = 0;
 
@@ -626,7 +624,7 @@ int status;
         name = "{}_test_allocate".format(self.__class__.__name__)
         source = r"""
 #include <stdio.h>
-#include "numba/runtime/nrt_external.h"
+#include "numba/core/runtime/nrt_external.h"
 
 NRT_MemInfo* test_nrt_api(NRT_api_functions *nrt, size_t n) {
     size_t *data = NULL;
