@@ -1,15 +1,13 @@
-from __future__ import print_function
-
 import collections
 import sys
 import weakref
 import gc
 
-import numba.unittest_support as unittest
-from numba.controlflow import CFGraph, Loop
-from numba.compiler import compile_extra, compile_isolated, Flags
-from numba import types
-from .support import TestCase
+import unittest
+from numba.core.controlflow import CFGraph, Loop
+from numba.core.compiler import compile_extra, compile_isolated, Flags
+from numba.core import types
+from numba.tests.support import TestCase
 
 enable_pyobj_flags = Flags()
 enable_pyobj_flags.set("enable_pyobject")
