@@ -1,5 +1,3 @@
-from __future__ import print_function
-
 import copy
 import itertools
 import operator
@@ -8,14 +6,14 @@ import warnings
 
 import numpy as np
 
-import numba.unittest_support as unittest
-from numba.compiler import compile_isolated, Flags
-from numba import jit, types, typeinfer, utils, errors
-from numba.config import PYVERSION
-from .support import TestCase, tag
-from .true_div_usecase import truediv_usecase, itruediv_usecase
-from .matmul_usecase import (matmul_usecase, imatmul_usecase, DumbMatrix,
-                             needs_matmul, needs_blas)
+import unittest
+from numba.core.compiler import compile_isolated, Flags
+from numba import jit
+from numba.core import types, utils, errors, typeinfer
+from numba.core.types.functions import _header_lead
+from numba.tests.support import TestCase, tag, needs_blas
+from numba.tests.matmul_usecase import (matmul_usecase, imatmul_usecase,
+                                        DumbMatrix,)
 
 Noflags = Flags()
 
@@ -58,15 +56,6 @@ class LiteralOperatorImpl(object):
         x *= y
         return x
 
-    @staticmethod
-    def div_usecase(x, y):
-        return x / y
-
-    @staticmethod
-    def idiv_usecase(x, y):
-        x /= y
-        return x
-
     @staticmethod
     def floordiv_usecase(x, y):
         return x // y
@@ -76,8 +65,15 @@ class LiteralOperatorImpl(object):
         x //= y
         return x
 
-    truediv_usecase = staticmethod(truediv_usecase)
-    itruediv_usecase = staticmethod(itruediv_usecase)
+    @staticmethod
+    def truediv_usecase(x, y):
+        return x / y
+
+    @staticmethod
+    def itruediv_usecase(x, y):
+        x /= y
+        return x
+
     if matmul_usecase:
         matmul_usecase = staticmethod(matmul_usecase)
         imatmul_usecase = staticmethod(imatmul_usecase)
@@ -197,6 +193,10 @@ class LiteralOperatorImpl(object):
     def not_in_usecase(x, y):
         return x not in y
 
+    @staticmethod
+    def is_usecase(x, y):
+        return x is y
+
 
 class FunctionalOperatorImpl(object):
 
@@ -224,18 +224,6 @@ class FunctionalOperatorImpl(object):
     def imul_usecase(x, y):
         return operator.imul(x, y)
 
-    if PYVERSION >= (3, 0):
-        div_usecase = NotImplemented
-        idiv_usecase = NotImplemented
-    else:
-        @staticmethod
-        def div_usecase(x, y):
-            return operator.div(x, y)
-
-        @staticmethod
-        def idiv_usecase(x, y):
-            return operator.idiv(x, y)
-
     @staticmethod
     def floordiv_usecase(x, y):
         return operator.floordiv(x, y)
@@ -368,6 +356,10 @@ class FunctionalOperatorImpl(object):
     def not_in_usecase(x, y):
         return not operator.contains(y, x)
 
+    @staticmethod
+    def is_usecase(x, y):
+        return operator.is_(x, y)
+
 
 class TestOperators(TestCase):
     """
@@ -394,8 +386,6 @@ class TestOperators(TestCase):
 
     def run_test_ints(self, pyfunc, x_operands, y_operands, types_list,
                       flags=force_pyobj_flags):
-        if pyfunc is NotImplemented:
-            self.skipTest("test irrelevant on this version of Python")
         for arg_types in types_list:
             cr = compile_isolated(pyfunc, arg_types, flags=flags)
             cfunc = cr.entry_point
@@ -417,8 +407,6 @@ class TestOperators(TestCase):
 
     def run_test_floats(self, pyfunc, x_operands, y_operands, types_list,
                         flags=force_pyobj_flags):
-        if pyfunc is NotImplemented:
-            self.skipTest("test irrelevant on this version of Python")
         for arg_types in types_list:
             cr = compile_isolated(pyfunc, arg_types, flags=flags)
             cfunc = cr.entry_point
@@ -478,45 +466,57 @@ class TestOperators(TestCase):
     def test_lt_scalar(self, flags=force_pyobj_flags):
         self.run_test_scalar_compare(self.op.lt_usecase, flags)
 
-    @tag('important')
     def test_lt_scalar_npm(self):
         self.test_lt_scalar(flags=Noflags)
 
     def test_le_scalar(self, flags=force_pyobj_flags):
         self.run_test_scalar_compare(self.op.le_usecase, flags)
 
-    @tag('important')
     def test_le_scalar_npm(self):
         self.test_le_scalar(flags=Noflags)
 
     def test_gt_scalar(self, flags=force_pyobj_flags):
         self.run_test_scalar_compare(self.op.gt_usecase, flags)
 
-    @tag('important')
     def test_gt_scalar_npm(self):
         self.test_gt_scalar(flags=Noflags)
 
     def test_ge_scalar(self, flags=force_pyobj_flags):
         self.run_test_scalar_compare(self.op.ge_usecase, flags)
 
-    @tag('important')
     def test_ge_scalar_npm(self):
         self.test_ge_scalar(flags=Noflags)
 
     def test_eq_scalar(self, flags=force_pyobj_flags):
         self.run_test_scalar_compare(self.op.eq_usecase, flags, ordered=False)
 
-    @tag('important')
     def test_eq_scalar_npm(self):
         self.test_eq_scalar(flags=Noflags)
 
     def test_ne_scalar(self, flags=force_pyobj_flags):
         self.run_test_scalar_compare(self.op.ne_usecase, flags, ordered=False)
 
-    @tag('important')
     def test_ne_scalar_npm(self):
         self.test_ne_scalar(flags=Noflags)
 
+    def test_is_ellipsis(self):
+        cres = compile_isolated(self.op.is_usecase, (types.ellipsis, types.ellipsis))
+        cfunc = cres.entry_point
+        self.assertTrue(cfunc(Ellipsis, Ellipsis))
+
+    def test_is_void_ptr(self):
+        # can't call this directly from python, as void cannot be unboxed
+        cfunc_void = jit(
+            (types.voidptr, types.voidptr), nopython=True
+        )(self.op.is_usecase)
+
+        # this wrapper performs the casts from int to voidptr for us
+        @jit(nopython=True)
+        def cfunc(x, y):
+            return cfunc_void(x, y)
+
+        self.assertTrue(cfunc(1, 1))
+        self.assertFalse(cfunc(1, 2))
 
     #
     # Arithmetic operators
@@ -613,7 +613,7 @@ class TestOperators(TestCase):
                           })
 
     generate_binop_tests(locals(),
-                         ('div', 'idiv', 'truediv', 'itruediv'),
+                         ('truediv', 'itruediv'),
                          {'ints': 'run_binop_ints',
                           'floats': 'run_binop_floats',
                           'complex': 'run_binop_complex',
@@ -629,8 +629,6 @@ class TestOperators(TestCase):
     def check_div_errors(self, usecase_name, msg, flags=force_pyobj_flags,
                          allow_complex=False):
         pyfunc = getattr(self.op, usecase_name)
-        if pyfunc is NotImplemented:
-            self.skipTest("%r not implemented" % (usecase_name,))
         # Signed and unsigned division can take different code paths,
         # test them both.
         arg_types = [types.int32, types.uint32, types.float64]
@@ -658,12 +656,6 @@ class TestOperators(TestCase):
     def test_floordiv_errors_npm(self):
         self.test_floordiv_errors(flags=Noflags)
 
-    def test_div_errors(self, flags=force_pyobj_flags):
-        self.check_div_errors("div_usecase", "division by zero", flags=flags)
-
-    def test_div_errors_npm(self):
-        self.test_div_errors(flags=Noflags)
-
     def test_mod_errors(self, flags=force_pyobj_flags):
         self.check_div_errors("mod_usecase", "modulo by zero", flags=flags)
 
@@ -728,7 +720,6 @@ class TestOperators(TestCase):
         self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
                              flags=flags)
 
-    @tag('important')
     def test_add_complex_npm(self):
         self.test_add_complex(flags=Noflags)
 
@@ -744,7 +735,6 @@ class TestOperators(TestCase):
         self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
                              flags=flags)
 
-    @tag('important')
     def test_sub_complex_npm(self):
         self.test_sub_complex(flags=Noflags)
 
@@ -760,26 +750,9 @@ class TestOperators(TestCase):
         self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
                              flags=flags)
 
-    @tag('important')
     def test_mul_complex_npm(self):
         self.test_mul_complex(flags=Noflags)
 
-    def test_div_complex(self, flags=force_pyobj_flags):
-        pyfunc = self.op.div_usecase
-
-        x_operands = [1+0j, 1j, -1-1j]
-        y_operands = [1, 2, 3]
-
-        types_list = [(types.complex64, types.complex64),
-                      (types.complex128, types.complex128),]
-
-        self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
-                             flags=flags)
-
-    @tag('important')
-    def test_div_complex_npm(self):
-        self.test_div_complex(flags=Noflags)
-
     def test_truediv_complex(self, flags=force_pyobj_flags):
         pyfunc = self.op.truediv_usecase
 
@@ -792,7 +765,6 @@ class TestOperators(TestCase):
         self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
                              flags=flags)
 
-    @tag('important')
     def test_truediv_complex_npm(self):
         self.test_truediv_complex(flags=Noflags)
 
@@ -801,7 +773,6 @@ class TestOperators(TestCase):
         with self.assertTypingError():
             cres = compile_isolated(pyfunc, (types.complex64, types.complex64))
 
-    @tag('important')
     def test_mod_complex_npm(self):
         self.test_mod_complex(flags=Noflags)
 
@@ -810,10 +781,8 @@ class TestOperators(TestCase):
     # (just check with simple values; computational tests are in test_linalg)
     #
 
-    @needs_matmul
     def check_matmul_objmode(self, pyfunc, inplace):
-        # Use dummy objects, to work with any Numpy / Scipy version
-        # (and because Numpy 1.10 doesn't implement "@=")
+        # Use dummy objects, to work with any NumPy / SciPy version
         cres = compile_isolated(pyfunc, (), flags=force_pyobj_flags)
         cfunc = cres.entry_point
         a = DumbMatrix(3)
@@ -826,16 +795,13 @@ class TestOperators(TestCase):
             self.assertIsNot(got, a)
             self.assertIsNot(got, b)
 
-    @needs_matmul
     def test_matmul(self):
         self.check_matmul_objmode(self.op.matmul_usecase, inplace=False)
 
-    @needs_matmul
     def test_imatmul(self):
         self.check_matmul_objmode(self.op.imatmul_usecase, inplace=True)
 
     @needs_blas
-    @needs_matmul
     def check_matmul_npm(self, pyfunc):
         arrty = types.Array(types.float32, 1, 'C')
         cres = compile_isolated(pyfunc, (arrty, arrty), flags=Noflags)
@@ -848,13 +814,9 @@ class TestOperators(TestCase):
         self.assertIsNot(got, a)
         self.assertIsNot(got, b)
 
-    @tag('important')
-    @needs_matmul
     def test_matmul_npm(self):
         self.check_matmul_npm(self.op.matmul_usecase)
 
-    @tag('important')
-    @needs_matmul
     def test_imatmul_npm(self):
         with self.assertTypingError() as raises:
             self.check_matmul_npm(self.op.imatmul_usecase)
@@ -1032,7 +994,6 @@ class TestOperators(TestCase):
         for val in values:
             self.assertPreciseEqual(pyfunc(val), cfunc(val))
 
-    @tag('important')
     def test_bitwise_not_npm(self):
         self.test_bitwise_not(flags=Noflags)
 
@@ -1045,7 +1006,7 @@ class TestOperators(TestCase):
             with self.assertRaises(errors.TypingError, msg=msg) as raises:
                 compile_isolated(pyfunc, argtypes)
             # check error message
-            fmt = 'Invalid use of {}'
+            fmt = _header_lead + ' {}'
             expecting = fmt.format(opname
                                    if isinstance(opname, str)
                                    else 'Function({})'.format(opname))
@@ -1086,7 +1047,6 @@ class TestOperators(TestCase):
         for val in values:
             self.assertEqual(pyfunc(val), cfunc(val))
 
-    @tag('important')
     def test_not_npm(self):
         pyfunc = self.op.not_usecase
         # test native mode
@@ -1112,7 +1072,6 @@ class TestOperators(TestCase):
 
     # XXX test_negate should check for negative and positive zeros and infinities
 
-    @tag('important')
     def test_negate_npm(self):
         pyfunc = self.op.negate_usecase
         # test native mode
@@ -1289,10 +1248,7 @@ class TestMixedInts(TestCase):
         return control_unsigned
 
     def run_binary(self, pyfunc, control_func, operands, types,
-                   expected_type=utils.INT_TYPES, **assertPreciseEqualArgs):
-        if pyfunc is NotImplemented:
-            self.skipTest("test irrelevant on this version of Python")
-
+                   expected_type=int, **assertPreciseEqualArgs):
         for xt, yt in types:
             cr = compile_isolated(pyfunc, (xt, yt), flags=Noflags)
             cfunc = cr.entry_point
@@ -1309,10 +1265,7 @@ class TestMixedInts(TestCase):
                                         **assertPreciseEqualArgs)
 
     def run_unary(self, pyfunc, control_func, operands, types,
-                  expected_type=utils.INT_TYPES):
-        if pyfunc is NotImplemented:
-            self.skipTest("test irrelevant on this version of Python")
-
+                  expected_type=int):
         for xt in types:
             cr = compile_isolated(pyfunc, (xt,), flags=Noflags)
             cfunc = cr.entry_point
@@ -1327,21 +1280,18 @@ class TestMixedInts(TestCase):
                         % (x, xt, got, expected))
 
     def run_arith_binop(self, pyfunc, opname, samples,
-                        expected_type=utils.INT_TYPES):
+                        expected_type=int):
         self.run_binary(pyfunc, self.get_control_signed(opname),
                         samples, self.signed_pairs, expected_type)
         self.run_binary(pyfunc, self.get_control_unsigned(opname),
                         samples, self.unsigned_pairs, expected_type)
 
-    @tag('important')
     def test_add(self):
         self.run_arith_binop(self.op.add_usecase, 'add', self.int_samples)
 
-    @tag('important')
     def test_sub(self):
         self.run_arith_binop(self.op.sub_usecase, 'sub', self.int_samples)
 
-    @tag('important')
     def test_mul(self):
         self.run_arith_binop(self.op.mul_usecase, 'mul', self.int_samples)
 
@@ -1378,8 +1328,9 @@ class TestMixedInts(TestCase):
                         samples, signed_pairs)
 
     def test_truediv(self):
+
         def control(a, b):
-            return truediv_usecase(float(a), float(b))
+            return float(a) / float(b)
         samples = [x for x in self.int_samples if x != 0]
         pyfunc = self.op.truediv_usecase
 
@@ -1554,5 +1505,99 @@ class TestStringConstComparison(TestCase):
         self.assertEqual(test_impl1(), cfunc1())
         self.assertEqual(test_impl2(), cfunc2())
 
+class TestBooleanLiteralOperators(TestCase):
+    """
+    Test operators with Boolean constants
+    """
+    def test_eq(self):
+
+        def test_impl1(b):
+            return a_val == b
+
+        def test_impl2(a):
+            return a == b_val
+
+        def test_impl3():
+            r1 = True == True
+            r2 = True == False
+            r3 = False == True
+            r4 = False == False
+            return (r1, r2, r3, r4)
+
+        for a_val, b in itertools.product([True, False], repeat=2):
+            cfunc1 = jit(nopython=True)(test_impl1)
+            self.assertEqual(test_impl1(b), cfunc1(b))
+
+        for a, b_val in itertools.product([True, False], repeat=2):
+            cfunc2 = jit(nopython=True)(test_impl2)
+            self.assertEqual(test_impl2(a), cfunc2(a))
+
+        cfunc3 = jit(nopython=True)(test_impl3)
+        self.assertEqual(test_impl3(), cfunc3())
+
+    def test_ne(self):
+
+        def test_impl1(b):
+            return a_val != b
+
+        def test_impl2(a):
+            return a != b_val
+
+        def test_impl3():
+            r1 = True != True
+            r2 = True != False
+            r3 = False != True
+            r4 = False != False
+            return (r1, r2, r3, r4)
+
+        for a_val, b in itertools.product([True, False], repeat=2):
+            cfunc1 = jit(nopython=True)(test_impl1)
+            self.assertEqual(test_impl1(b), cfunc1(b))
+
+        for a, b_val in itertools.product([True, False], repeat=2):
+            cfunc2 = jit(nopython=True)(test_impl2)
+            self.assertEqual(test_impl2(a), cfunc2(a))
+
+        cfunc3 = jit(nopython=True)(test_impl3)
+        self.assertEqual(test_impl3(), cfunc3())
+
+    def test_is(self):
+
+        def test_impl1(b):
+            return a_val is b
+
+        def test_impl2():
+            r1 = True is True
+            r2 = True is False
+            r3 = False is True
+            r4 = False is False
+            return (r1, r2, r3, r4)
+
+        for a_val, b in itertools.product([True, False], repeat=2):
+            cfunc1 = jit(nopython=True)(test_impl1)
+            self.assertEqual(test_impl1(b), cfunc1(b))
+
+        cfunc2 = jit(nopython=True)(test_impl2)
+        self.assertEqual(test_impl2(), cfunc2())
+
+    def test_not(self):
+
+        def test_impl():
+            a, b = False, True
+            return (not a, not b)
+
+        cfunc = jit(nopython=True)(test_impl)
+        self.assertEqual(test_impl(), cfunc())
+
+    def test_bool(self):
+
+        def test_impl():
+            a, b = False, True
+            return (bool(a), bool(b))
+
+        cfunc = jit(nopython=True)(test_impl)
+        self.assertEqual(test_impl(), cfunc())
+
+
 if __name__ == '__main__':
     unittest.main()
