@@ -1,20 +1,21 @@
-from __future__ import print_function, division, absolute_import
-
 import collections
 import itertools
 
 import numpy as np
 
-from numba import unittest_support as unittest
-from numba.compiler import compile_isolated
-from numba import njit, jit, types, errors, utils
-from .support import TestCase, MemoryLeakMixin, tag
+from numba.core.compiler import compile_isolated
+from numba import njit, jit, typeof, literally
+from numba.core import types, errors, utils
+from numba.tests.support import TestCase, MemoryLeakMixin, tag
+import unittest
 
 
 Rect = collections.namedtuple('Rect', ('width', 'height'))
 
 Point = collections.namedtuple('Point', ('x', 'y', 'z'))
 
+Point2 = collections.namedtuple('Point2', ('x', 'y', 'z'))
+
 Empty = collections.namedtuple('Empty', ())
 
 def tuple_return_usecase(a, b):
@@ -143,7 +144,6 @@ class TestTupleTypeNotIterable(unittest.TestCase):
 
 class TestTupleReturn(TestCase):
 
-    @tag('important')
     def test_array_tuple(self):
         aryty = types.Array(types.float64, 1, 'C')
         cres = compile_isolated(tuple_return_usecase, (aryty, aryty))
@@ -162,7 +162,6 @@ class TestTupleReturn(TestCase):
         self.assertEqual(ra, a)
         self.assertEqual(rb, b)
 
-    @tag('important')
     def test_hetero_tuple(self):
         alltypes = []
         allvalues = []
@@ -184,7 +183,6 @@ class TestTupleReturn(TestCase):
 
 class TestTuplePassing(TestCase):
 
-    @tag('important')
     def test_unituple(self):
         tuple_type = types.UniTuple(types.int32, 2)
         cr_first = compile_isolated(tuple_first, (tuple_type,))
@@ -192,7 +190,6 @@ class TestTuplePassing(TestCase):
         self.assertPreciseEqual(cr_first.entry_point((4, 5)), 4)
         self.assertPreciseEqual(cr_second.entry_point((4, 5)), 5)
 
-    @tag('important')
     def test_hetero_tuple(self):
         tuple_type = types.Tuple((types.int64, types.float32))
         cr_first = compile_isolated(tuple_first, (tuple_type,))
@@ -212,7 +209,6 @@ class TestTuplePassing(TestCase):
 
 class TestOperations(TestCase):
 
-    @tag('important')
     def test_len(self):
         pyfunc = len_usecase
         cr = compile_isolated(pyfunc,
@@ -222,7 +218,17 @@ class TestOperations(TestCase):
                               [types.UniTuple(types.int64, 3)])
         self.assertPreciseEqual(cr.entry_point((4, 5, 6)), 3)
 
-    @tag('important')
+    def test_index_literal(self):
+        # issue #6023, test non-static getitem with IntegerLiteral index
+        def pyfunc(tup, idx):
+            idx = literally(idx)
+            return tup[idx]
+        cfunc = njit(pyfunc)
+
+        tup = (4, 3.1, 'sss')
+        for i in range(len(tup)):
+            self.assertPreciseEqual(cfunc(tup, i), tup[i])
+
     def test_index(self):
         pyfunc = tuple_index
         cr = compile_isolated(pyfunc,
@@ -273,7 +279,7 @@ class TestOperations(TestCase):
         with self.assertTypingError() as raises:
             cr = compile_isolated(pyfunc, ())
         msg = ("Cannot infer the type of variable 'c', have imprecise type: "
-               "list(undefined).")
+               "list(undefined)<iv=None>.")
         self.assertIn(msg, str(raises.exception))
 
 
@@ -315,7 +321,6 @@ class TestOperations(TestCase):
                               [types.Tuple(())])
         self.assertPreciseEqual(cr.entry_point(()), pyfunc(()))
 
-    @tag('important')
     def test_add(self):
         pyfunc = add_usecase
         samples = [(types.Tuple(()), ()),
@@ -355,27 +360,21 @@ class TestOperations(TestCase):
                      ((4, 5), (4, 6, 7))]:
             eq(pyfunc, cfunc, args)
 
-    @tag('important')
     def test_eq(self):
         self._test_compare(eq_usecase)
 
-    @tag('important')
     def test_ne(self):
         self._test_compare(ne_usecase)
 
-    @tag('important')
     def test_gt(self):
         self._test_compare(gt_usecase)
 
-    @tag('important')
     def test_ge(self):
         self._test_compare(ge_usecase)
 
-    @tag('important')
     def test_lt(self):
         self._test_compare(lt_usecase)
 
-    @tag('important')
     def test_le(self):
         self._test_compare(le_usecase)
 
@@ -450,11 +449,9 @@ class TestNamedTuple(TestCase, MemoryLeakMixin):
                      ((4, 5), (4, 6, 7))]:
             eq(pyfunc, cfunc, (Rect(*a), Point(*b)))
 
-    @tag('important')
     def test_eq(self):
         self._test_compare(eq_usecase)
 
-    @tag('important')
     def test_ne(self):
         self._test_compare(ne_usecase)
 
@@ -470,7 +467,6 @@ class TestNamedTuple(TestCase, MemoryLeakMixin):
     def test_le(self):
         self._test_compare(le_usecase)
 
-    @tag('important')
     def test_getattr(self):
         pyfunc = getattr_usecase
         cfunc = jit(nopython=True)(pyfunc)
@@ -479,7 +475,6 @@ class TestNamedTuple(TestCase, MemoryLeakMixin):
             p = Point(*args)
             self.assertPreciseEqual(cfunc(p), pyfunc(p))
 
-    @tag('important')
     def test_construct(self):
         def check(pyfunc):
             cfunc = jit(nopython=True)(pyfunc)
@@ -528,7 +523,6 @@ class TestNamedTuple(TestCase, MemoryLeakMixin):
         r = call(123, 0)
         self.assertEqual(r, Rect(width=123, height=-321))
 
-    @unittest.skipIf(utils.PYVERSION < (3, 0), "needs Python 3")
     def test_string_literal_in_ctor(self):
         # Test for issue #3813
 
@@ -539,6 +533,32 @@ class TestNamedTuple(TestCase, MemoryLeakMixin):
         r = foo()
         self.assertEqual(r, Rect(width=10, height='somestring'))
 
+    def test_dispatcher_mistreat(self):
+        # Test for issue #5215 that mistreat namedtuple as tuples
+        @jit(nopython=True)
+        def foo(x):
+            return x
+
+        in1 = (1, 2,  3)
+        out1 = foo(in1)
+        self.assertEqual(in1, out1)
+
+        in2 = Point(1, 2, 3)
+        out2 = foo(in2)
+        self.assertEqual(in2, out2)
+
+        # Check the signatures
+        self.assertEqual(len(foo.nopython_signatures), 2)
+        self.assertEqual(foo.nopython_signatures[0].args[0], typeof(in1))
+        self.assertEqual(foo.nopython_signatures[1].args[0], typeof(in2))
+
+        # Differently named
+        in3 = Point2(1, 2, 3)
+        out3 = foo(in3)
+        self.assertEqual(in3, out3)
+        self.assertEqual(len(foo.nopython_signatures), 3)
+        self.assertEqual(foo.nopython_signatures[2].args[0], typeof(in3))
+
 
 class TestTupleNRT(TestCase, MemoryLeakMixin):
     def test_tuple_add(self):
@@ -611,7 +631,6 @@ class TestMethods(TestCase):
 
 class TestTupleBuild(TestCase):
 
-    @unittest.skipIf(utils.PYVERSION < (3, 0), "needs Python 3")
     def test_build_unpack(self):
         def check(p):
             # using eval here since Python 2 doesn't even support the syntax
@@ -624,8 +643,6 @@ class TestTupleBuild(TestCase):
         # Heterogeneous
         check((4, 5.5))
 
-
-    @unittest.skipIf(utils.PYVERSION < (3, 0), "needs Python 3")
     def test_build_unpack_more(self):
         def check(p):
             # using eval here since Python 2 doesn't even support the syntax
@@ -638,8 +655,6 @@ class TestTupleBuild(TestCase):
         # Heterogeneous
         check((4, 5.5))
 
-
-    @unittest.skipIf(utils.PYVERSION < (3, 0), "needs Python 3")
     def test_build_unpack_call(self):
         def check(p):
             # using eval here since Python 2 doesn't even support the syntax
