@@ -1,14 +1,11 @@
-from __future__ import print_function, unicode_literals
-
-import platform
 import numpy as np
 
-import numba.unittest_support as unittest
-from numba import jit, utils, from_dtype, types
+import unittest
+from numba import jit, from_dtype
+from numba.core import types, utils
 from numba.typed import Dict
-from numba.tests.support import TestCase
+from numba.tests.support import (TestCase, skip_ppc64le_issue4563)
 
-skip_py2 = unittest.skipIf(not utils.IS_PY3, "not supported in Python 2")
 require_py37 = unittest.skipIf(utils.PYVERSION < (3, 7), "requires Python 3.7+")
 
 
@@ -219,8 +216,7 @@ def return_not(x, i):
     return not x[i]
 
 
-@skip_py2
-@unittest.skipIf(platform.machine() == 'ppc64le', "LLVM bug")
+@skip_ppc64le_issue4563
 class TestUnicodeArray(TestCase):
 
     def _test(self, pyfunc, cfunc, *args, **kwargs):
