@@ -1,14 +1,11 @@
 """
 Compiler-side implementation of the Numba  typed-list.
 """
-import ctypes
 import operator
 from enum import IntEnum
 
 from llvmlite import ir
 
-from numba import _helperlib
-
 from numba.core.extending import (
     overload,
     overload_method,
@@ -69,8 +66,9 @@ class ListModel(models.StructModel):
 class ListIterModel(models.StructModel):
     def __init__(self, dmm, fe_type):
         members = [
-            ('parent', fe_type.parent),  # reference to the list
-            ('state', types.voidptr),    # iterator state in C code
+            ('size', types.intp), # the size of the iteration space
+            ('parent', fe_type.parent), # the parent list
+            ('index', types.EphemeralPointer(types.intp)), # current index
         ]
         super(ListIterModel, self).__init__(dmm, fe_type, members)
 
@@ -416,21 +414,24 @@ def _list_length(typingctx, l):
 
     Returns the length of the list.
     """
-    resty = types.intp
-    sig = resty(l)
+    sig = types.intp(l)
 
     def codegen(context, builder, sig, args):
+        [tl] = sig.args
+        [l] = args
         fnty = ir.FunctionType(
             ll_ssize_t,
             [ll_list_type],
         )
-        fn = builder.module.get_or_insert_function(fnty,
-                                                   name='numba_list_length')
-        [l] = args
-        [tl] = sig.args
+        fname = 'numba_list_size_address'
+        fn = builder.module.get_or_insert_function(fnty, name=fname)
+        fn.attributes.add('alwaysinline')
+        fn.attributes.add('readonly')
+        fn.attributes.add('nounwind')
         lp = _container_get_data(context, builder, tl, l)
-        n = builder.call(fn, [lp])
-        return n
+        len_addr = builder.call(fn, [lp,],)
+        ptr = builder.inttoptr(len_addr, cgutils.intp_t.as_pointer())
+        return builder.load(ptr)
 
     return sig, codegen
 
@@ -618,6 +619,33 @@ def impl_append(l, item):
         return sig, impl
 
 
+@intrinsic
+def fix_index(tyctx, list_ty, index_ty):
+    sig = types.intp(list_ty, index_ty)
+
+    def codegen(context, builder, sig, args):
+        [list_ty, index_ty] = sig.args
+        [ll_list, ll_idx] = args
+        is_negative = builder.icmp_signed('<', ll_idx,
+                                          ir.Constant(ll_idx.type, 0))
+        fast_len_sig, length_fn = _list_length._defn(context.typing_context,
+                                                     list_ty)
+        length = length_fn(context, builder, fast_len_sig, (ll_list,))
+        # length is an intp
+        # index can be any sort of int
+        # indexing in general is done with a ssize_t which correlates to an
+        # intp. In llvmlite sext and trunc are guarded to return the value
+        # itself if the types are the same, so there's no need to handle the
+        # "equal widths" case separately. This sexts/truncs the index to the
+        # length type such that `add` works for the wraparound case.
+        st = 'sext' if ll_idx.type.width < length.type.width else 'trunc'
+        op = getattr(builder, st)
+        fixedup_idx = op(ll_idx, length.type)
+        wrapped_index = builder.add(fixedup_idx, length)
+        return builder.select(is_negative, wrapped_index, fixedup_idx)
+    return sig, codegen
+
+
 @register_jitable
 def handle_index(l, index):
     """Handle index.
@@ -626,12 +654,9 @@ def handle_index(l, index):
     an IndexError.
     """
     # convert negative indices to positive ones
-    if index < 0:
-        # len(l) has always type 'int64'
-        # while index can be an signed/unsigned integer
-        index = type(index)(len(l) + index)
+    index = fix_index(l, index)
     # check that the index is in range
-    if not (0 <= index < len(l)):
+    if index < 0 or index >= len(l):
         raise IndexError("list index out of range")
     return index
 
@@ -659,64 +684,70 @@ def handle_slice(l, s):
     return range(start, stop, s.step)
 
 
-@intrinsic
-def _list_getitem(typingctx, l, index):
-    """Wrap numba_list_getitem
+def _gen_getitem(borrowed):
 
-    Returns 2-tuple of (int32, ?item_type)
+    @intrinsic
+    def impl(typingctx, l_ty, index_ty):
 
-    """
-    IS_NOT_NONE = not isinstance(l.item_type, types.NoneType)
-    resty = types.Tuple([types.int32,
-                         types.Optional(l.item_type
-                                        if IS_NOT_NONE
-                                        else types.int64)])
-    sig = resty(l, index)
+        is_none = isinstance(l_ty.item_type, types.NoneType)
+        if is_none:
+            resty = types.Tuple([types.int32, l_ty.item_type])
+        else:
+            resty = types.Tuple([types.int32, types.Optional(l_ty.item_type)])
+        sig = resty(l_ty, index_ty)
 
-    def codegen(context, builder, sig, args):
-        fnty = ir.FunctionType(
-            ll_status,
-            [ll_list_type, ll_ssize_t, ll_bytes],
-        )
-        [tl, tindex] = sig.args
-        [l, index] = args
-        fn = builder.module.get_or_insert_function(
-            fnty, name='numba_list_getitem')
+        def codegen(context, builder, sig, args):
+            [tl, tindex] = sig.args
+            [l, index] = args
+            fnty = ir.FunctionType(
+                ll_voidptr_type,
+                [ll_list_type],
+            )
+            fname = 'numba_list_base_ptr'
+            fn = builder.module.get_or_insert_function(fnty, fname)
+            fn.attributes.add('alwaysinline')
+            fn.attributes.add('nounwind')
+            fn.attributes.add('readonly')
 
-        dm_item = context.data_model_manager[tl.item_type]
-        ll_item = context.get_data_type(tl.item_type)
-        ptr_item = cgutils.alloca_once(builder, ll_item)
+            lp = _container_get_data(context, builder, tl, l)
 
-        lp = _container_get_data(context, builder, tl, l)
-        status = builder.call(
-            fn,
-            [
-                lp,
-                index,
-                _as_bytes(builder, ptr_item),
-            ],
-        )
-        # Load item if output is available
-        found = builder.icmp_signed('>=', status,
-                                    status.type(int(ListStatus.LIST_OK)))
-        out = context.make_optional_none(builder,
-                                         tl.item_type
-                                         if IS_NOT_NONE
-                                         else types.int64)
-        pout = cgutils.alloca_once_value(builder, out)
+            base_ptr = builder.call(
+                fn,
+                [lp,],
+            )
 
-        with builder.if_then(found):
-            if IS_NOT_NONE:
-                item = dm_item.load_from_data_pointer(builder, ptr_item)
-                context.nrt.incref(builder, tl.item_type, item)
-                loaded = context.make_optional_value(
-                    builder, tl.item_type, item)
+            llty = context.get_data_type(tl.item_type)
+            casted_base_ptr = builder.bitcast(base_ptr, llty.as_pointer())
+
+            item_ptr = cgutils.gep(builder, casted_base_ptr, index)
+
+            if is_none:
+                out = builder.load(item_ptr)
+            else:
+                out = context.make_optional_none(builder, tl.item_type)
+                pout = cgutils.alloca_once_value(builder, out)
+
+                dm_item = context.data_model_manager[tl.item_type]
+                item = dm_item.load_from_data_pointer(builder, item_ptr)
+                if not borrowed:
+                    context.nrt.incref(builder, tl.item_type, item)
+
+                if is_none:
+                    loaded = item
+                else:
+                    loaded = context.make_optional_value(builder, tl.item_type,
+                                                         item)
                 builder.store(loaded, pout)
 
-        out = builder.load(pout)
-        return context.make_tuple(builder, resty, [status, out])
+                out = builder.load(pout)
+            return context.make_tuple(builder, resty, [ll_status(0), out])
 
-    return sig, codegen
+        return sig, codegen
+    return impl
+
+
+_list_getitem = _gen_getitem(False)
+_list_getitem_borrowed = _gen_getitem(True)
 
 
 @overload(operator.getitem)
@@ -1263,6 +1294,19 @@ def ol_list_sort(lst, key=None, reverse=False):
     return impl
 
 
+@overload_method(types.ListType, "getitem_unchecked")
+def ol_getitem_unchecked(lst, index):
+    if not isinstance(index, types.Integer):
+        return
+
+    def impl(lst, index):
+        index = fix_index(lst, index)
+        castedindex = _cast(index, types.intp)
+        _, item = _list_getitem(lst, castedindex)
+        return _nonoptional(item)
+    return impl
+
+
 @overload_attribute(types.ListType, '_dtype')
 def impl_dtype(l):
     if not isinstance(l, types.ListType):
@@ -1390,85 +1434,96 @@ def impl_greater_than_or_equal(this, other):
     return compare_helper(this, other, (0, 1))
 
 
+class ListIterInstance(object):
+
+    def __init__(self, context, builder, iter_type, iter_val):
+        self._context = context
+        self._builder = builder
+        self._iter_ty = iter_type
+        self._list_ty = self._iter_ty.parent
+        self._iter = context.make_helper(builder, iter_type, iter_val)
+
+    @classmethod
+    def from_list(cls, context, builder, iter_type, list_val):
+        self = cls(context, builder, iter_type, None)
+        index = context.get_constant(types.intp, 0)
+        self._iter.index = cgutils.alloca_once_value(builder, index)
+        self._iter.parent = list_val
+        self._iter.size = cls._size_of_list(context, builder, self._list_ty,
+                                            self._iter.parent)
+        return self
+
+    @classmethod
+    def _size_of_list(cls, context, builder, list_ty, ll_list):
+        tyctx = context.typing_context
+        fnty = tyctx.resolve_value_type(len)
+        sig = fnty.get_call_type(tyctx, (list_ty,), {})
+        impl = context.get_function(fnty, sig)
+        return impl(builder, (ll_list,))
+
+    @property
+    def size(self):
+        tyctx = self._context.typing_context
+        fnty = tyctx.resolve_value_type(len)
+        ty = self._list_ty
+        sig = fnty.get_call_type(tyctx, (ty,), {})
+        impl = self._context.get_function(fnty, sig)
+        return impl(self._builder, (self._iter.parent,))
+
+    @property
+    def value(self):
+        return self._iter._getvalue()
+
+    def getitem(self, index):
+        tyctx = self._context.typing_context
+        ty = self._list_ty
+        sig, fn = _list_getitem_borrowed._defn(tyctx, ty, types.intp)
+
+        statnitem = fn(self._context, self._builder, sig, (self._iter.parent,
+                                                           index))
+        _, item = cgutils.unpack_tuple(self._builder, statnitem)
+        retty = sig.return_type[1]
+        if isinstance(self._list_ty.dtype, types.NoneType):
+            raw_ty = self._list_ty.dtype
+        else:
+            raw_ty = retty.type
+        raw_item = self._context.cast(self._builder, item, retty, raw_ty)
+        return raw_item
+
+    @property
+    def index(self):
+        return self._builder.load(self._iter.index)
+
+    @index.setter
+    def index(self, value):
+        self._builder.store(value, self._iter.index)
+
+
 @lower_builtin('getiter', types.ListType)
-def impl_list_getiter(context, builder, sig, args):
-    """Implement iter(List).
-    """
-    [tl] = sig.args
-    [l] = args
-    iterablety = types.ListTypeIterableType(tl)
-    it = context.make_helper(builder, iterablety.iterator_type)
-
-    fnty = ir.FunctionType(
-        ir.VoidType(),
-        [ll_listiter_type, ll_list_type],
-    )
-
-    fn = builder.module.get_or_insert_function(fnty, name='numba_list_iter')
-
-    proto = ctypes.CFUNCTYPE(ctypes.c_size_t)
-    listiter_sizeof = proto(_helperlib.c_helpers['list_iter_sizeof'])
-    state_type = ir.ArrayType(ir.IntType(8), listiter_sizeof())
-
-    pstate = cgutils.alloca_once(builder, state_type, zfill=True)
-    it.state = _as_bytes(builder, pstate)
-    it.parent = l
-
-    dp = _container_get_data(context, builder, iterablety.parent, args[0])
-    builder.call(fn, [it.state, dp])
-    return impl_ret_borrowed(
-        context,
-        builder,
-        sig.return_type,
-        it._getvalue(),
-    )
+def getiter_list(context, builder, sig, args):
+    inst = ListIterInstance.from_list(context, builder, sig.return_type,
+                                      args[0])
+    return impl_ret_borrowed(context, builder, sig.return_type, inst.value)
 
 
 @lower_builtin('iternext', types.ListTypeIteratorType)
 @iternext_impl(RefType.BORROWED)
-def impl_iterator_iternext(context, builder, sig, args, result):
-    iter_type = sig.args[0]
-    it = context.make_helper(builder, iter_type, args[0])
+def iternext_listiter(context, builder, sig, args, result):
+    inst = ListIterInstance(context, builder, sig.args[0], args[0])
+    index = inst.index
 
-    iternext_fnty = ir.FunctionType(
-        ll_status,
-        [ll_listiter_type, ll_bytes.as_pointer()]
-    )
-    iternext = builder.module.get_or_insert_function(
-        iternext_fnty,
-        name='numba_list_iter_next',
-    )
-    item_raw_ptr = cgutils.alloca_once(builder, ll_bytes)
+    nitems = inst.size # this is current size
+    init_size = inst._iter.size # this is initial size
 
-    status = builder.call(iternext, (it.state, item_raw_ptr))
-
-    # check for list mutation
-    mutated_status = status.type(int(ListStatus.LIST_ERR_MUTATED))
-    is_mutated = builder.icmp_signed('==', status, mutated_status)
+    # if the current count is different to the initial count, bail, list is
+    # being mutated whilst iterated.
+    is_mutated = builder.icmp_signed('!=', init_size, nitems)
     with builder.if_then(is_mutated, likely=False):
         context.call_conv.return_user_exc(
             builder, RuntimeError, ("list was mutated during iteration",))
 
-    # if the list wasn't mutated it is either fine or the iterator was
-    # exhausted
-    ok_status = status.type(int(ListStatus.LIST_OK))
-    is_valid = builder.icmp_signed('==', status, ok_status)
+    is_valid = builder.icmp_signed('<', index, nitems)
     result.set_valid(is_valid)
-
-    with builder.if_then(is_valid, likely=True):
-        item_ty = iter_type.parent.item_type
-
-        dm_item = context.data_model_manager[item_ty]
-
-        item_ptr = builder.bitcast(
-            builder.load(item_raw_ptr),
-            dm_item.get_data_type().as_pointer(),
-        )
-
-        item = dm_item.load_from_data_pointer(builder, item_ptr)
-
-        if isinstance(iter_type.iterable, ListTypeIterableType):
-            result.yield_(item)
-        else:
-            # unreachable
-            raise AssertionError('unknown type: {}'.format(iter_type.iterable))
+    with builder.if_then(is_valid):
+        result.yield_(inst.getitem(index))
+        inst.index = builder.add(index, context.get_constant(types.intp, 1))
