@@ -436,11 +436,18 @@ def box_lsttype(typ, val, c):
 
     lsttype_obj = c.pyapi.unserialize(c.pyapi.serialize_object(typ))
 
-    res = c.pyapi.call_function_objargs(fmp_fn, (boxed_meminfo, lsttype_obj))
-    c.pyapi.decref(fmp_fn)
-    c.pyapi.decref(typedlist_mod)
-    c.pyapi.decref(boxed_meminfo)
-    return res
+    result_var = builder.alloca(c.pyapi.pyobj)
+    builder.store(cgutils.get_null_value(c.pyapi.pyobj), result_var)
+
+    with builder.if_then(cgutils.is_not_null(builder, lsttype_obj)):
+        res = c.pyapi.call_function_objargs(
+            fmp_fn, (boxed_meminfo, lsttype_obj),
+        )
+        c.pyapi.decref(fmp_fn)
+        c.pyapi.decref(typedlist_mod)
+        c.pyapi.decref(boxed_meminfo)
+        builder.store(res, result_var)
+    return builder.load(result_var)
 
 
 @unbox(types.ListType)
